/************************************************************************************/
/*                                                                                  */
/*     NAME: missing.mac.sas (fr die SAS Version 9.1)                              */
/*    TITLE: Missing Value-Ersetzung                                                */
/*     KEYS: missing, missing values, imputation, single imputation,                */
/*           multiple imputation, monotone missing pattern, arbitrary missing       */
/*           pattern, imputation model, distribution                                */
/*  VERSION: SAS 8.2, gendert auf SAS 9.1                                           */
/*  PRODUCT: BASE, STAT                                                             */
/*    PROCS: FREQ, STDIZE, MI                                                       */
/*                                                                                  */
/*                                                                                  */
/*  Verzeichnis/Dateiname:          F:\missing\missing.mac.sas                      */
/*  Autoren:                        R. Muche, C. Ring, C. Ziegler                   */
/*  Programmierer:                  Christoph Ziegler                               */
/*  Datum der Ersterstellung:       5/2003                                          */
/*  Aktuelle Makro-Version:         12/2003 gendert auf 08/2005                    */
/*                                                                                  */
/*  Kurzbeschreibung:               Makro zur Ersetzung von missing values mittels  */
/*                                  Single- und Multiple Imputation                 */
/*                                                                                  */
/*  Verwendete externe Untermakros: numberobs.mac.sas                               */
/*                                  exist.mac.sas                                   */
/*                                  exist_var.mac.sas                               */
/*                                  numchar.mac.sas                                 */
/*                                                                                  */
/*  Bentigte Dateien:              Datensatz (Parameter 'data=')                   */
/*                                                                                  */
/*                                                                                  */
/*  Erzeugte Dateien:               work.simputed                                   */
/*                                  (falls Parameter 'imputation_art' =1 oder =3)   */
/*  (bei Standardeinstellung;       work.mimputed1 - work.mimputedx                 */
/*  Dateinamen frei whlbar)        (x ist Einstellung von 'nimpute=')              */
/*                                  (falls Parameter 'imputation_art' =2 oder =3)   */
/*                                                                                  */
/*  Programmnderungen                                                              */ 
/*  und -ergnzungen                                                                */
/*  (Name, Datum, Beschreibung):    Kati Brodrecht,	Mai-August'05, siehe Kommentare */
/*                                  im Quellcode,									*/
/*									Auswahl einer neuen multiplen Ersetzungsmethode */
/*									(Predictive Mean Matching) fr fehlende Werte   */
/*									stetiger Variablen;								*/
/*                                  Mglichkeit der Ersetzung fehlender Werte von   */
/*                                  kategorialen Variablen mittels der Discriminant */
/*                                  Function Method bzw. Logistischen Regressions-	*/
/*                                  methode bei monotonem Missing Pattern, wenn     */
/*                                  mindestens eine volltndige stetige Variable als*/
/*                                  Kovariable bergeben wird;						*/
/*									Mglichkeit anzugeben, ob Minimum- und Maximum- */
/*									angaben innerhalb der Prozedur MI fr kategori-	*/
/*									ale Variablen in Verbindung mit dem urspr.		*/
/*									Ersetzungsmethoden verwendet werden sollen;     */										
/*									Ausgabe eines Missing Pattern mittels PROC MI   */
/*									ohne Ersetzung der fehlenden Werte im Hinter-	*/
/*									grund.  										*/
/*                                                                                  */
/************************************************************************************/
/*

 TITLE
 -----
 missing.mac.sas

 SAS Makro zur Ersetzung von missing values mittels Single- und Multiple Imputation


 BACKGROUND
 ----------

 Grnde fr fehlenden Werte:

 Das Fehlen von Beobachtungen kann verschiedene Ursachen haben. Ein Wert kann 
 "missing completely at random (MCAR)" sein, wenn die Wahrscheinlichkeit fr 
 das Fehlen von der Zielgre (z.B. im Zusammenhang mit Regressionsanalysen) 
 oder den anderen im Datensatz vorhandenen Variablen unabhngig ist. Diese 
 relativ einfach zu handhabende Situation ist aber in der Regel unrealistisch. 
 "Missing at random" (MAR) ist ein fehlender Wert, wenn dessen Fehlen nicht von 
 der Zielgre, mglicherweise aber von anderen Einflussgren abhngt. Am 
 problematischsten zu handhaben ist die dritte Mglichkeit fr das Auftreten eines 
 fehlenden Wertes, wenn dieser vom eigentlichen Wert des Fehlenden abhngt 
 ("non missing at random" (NMAR)) [scheffer2002]. Problematisch fr die 
 Modellierung ist der wohl wahrscheinlichste Fall in Bezug auf Regressions-
 analysen, wenn / falls der fehlende Wert von der Zielgre abhngt. 

 Problem fehlender Werte:

 Das Problem fehlender Werte ist, dass die Ergebnisse einer Analyse in der 
 Regel verzerrt werden, d.h. einen Bias haben. Bei vergleichsweise hohen 
 Anzahlen von fehlenden Werten kann die Interpretation der Ergebnisse nicht 
 eindeutig sein. Es kann u.U. ein Selektionsbias vorliegen, wenn die fehlenden 
 Werte gerade von Beobachtungen einer speziellen Gruppe stammen und so die 
 Reprsentativitt und Verallgemeinerbarkeit der Untersuchung in Frage 
 gestellt werden muss. 

 Complete Case Analyse:

 Im Rahmen der Complete Case Analyse bei der Regressionsanalyse wird der 
 Datensatz auf lediglich diejenigen Beobachtungen beschrnkt, die in den 
 Modellvariablen vollstndige Information haben. Die Complete Case Analyse hat 
 einige gute Eigenschaften, die ihre Anwendung rechtfertigen. Unter anderem 
 gehrt dazu die Einfachheit des Vorgehens. Speziell in der Situation, in der 
 nur wenige fehlende Werte vorhanden sind, ist eine Auswertung auf Basis der 
 vollstndigen Datenstze ausreichend. Es gibt aber einige Nachteile, die auf 
 der Hand liegen. Dazu gehrt die Reduktion der Fallzahl, was im besten Fall 
 bereits zu einem Powerverlust fhrt. Die Interpretierbarkeit der Ergebnisse 
 einer solchen Analyse ist eigentlich nur unter der Annahme gegeben, dass das 
 brig gebliebene Kollektiv fr den gesamten Datensatz reprsentativ ist. In 
 der Regel wird in der Realitt ein Selektionsbias erzeugt werden. Als ein 
 weiteres Problem bei der Modellierung in Datenstzen mit fehlenden Werten 
 erweist sich, dass bei Vergleichen von verschiedenen Modellen auch verschiedene 
 Fallzahlen vorliegen, da die SAS-Prozeduren jeweils die vollstndigen 
 Beobachtungen fr die im Modell angegebenen Variablen heranziehen. Durch die 
 Auswahl unterschiedlicher Variablen erzeugt man so unterschiedliche Datenstze, 
 wenn der Datensatz nicht vorher explizit reduziert wird (was die maximale 
 Fallzahlreduktion zur Folge hat). 

 Ersetzungsstrategien:

 Aufgrund der vielen Probleme im Zusammenhang mit einer Complete Case Analyse 
 sollte versucht werden, mit fehlenden Werten umzugehen und verschiedene 
 Ersetzungsstrategien (Imputation) vorzugeben. In den letzten Jahren sind dazu 
 viele Vorschlge gemacht und untersucht worden. Von Little und Rubin [little1987], 
 Rubin [rubin1987], Shafer [shafer1997] sowie von Allison [allison2001] gibt es 
 Standardlehrbcher zur Analyse von Daten mit fehlenden Werten. Vach hat hierzu 
 speziell einen Beitrag zur Analyse logistischer Regressionen mit fehlenden Werten 
 beigetragen [vach1994]. 
 Harrell gibt den Rat, auch wenn Imputationen mit Vorsicht anzuwenden sind, 
 dass es meist besser ist, die fehlenden Werte zu schtzen als die gesamte 
 Beobachtung nicht zu bercksichtigen [harrell2001].

 Single Imputation, Multiple Imputation:
  
 Bei der Single Imputation werden die fehlenden Werte in einer stetigen 
 Variablen im einfachsten Fall durch eine statistische Kenngre wie den 
 Median oder Mittelwert der vorhandenen Beobachtungen ersetzt. Ist bekannt, 
 dass die Werte der fehlenden Variablen von anderen Variablen abhngen, kann 
 auch ein Regressionsansatz zur Imputation herangezogen werden. Fr diskrete 
 Variablen kann man den Modus einsetzen oder eine zustzliche Kategorie 
 "Missing" einfhren. Dadurch ist allerdings jeweils ein zustzlicher 
 Regressionskoeffizient (im Zusammenhang mit Regressionsanalysen) im Modell zu
 schtzen. Der Vorteil bei der Single Imputation ist die Einfachheit des Vorgehens. 
 Es resultiert auch nur ein vervollstndigter Datensatz, der fr die Analyse zur 
 Verfgung steht. Die Hauptprobleme bei diesem Vorgehen bestehen in einem Bias bei 
 der Schtzung der Regressionskoeffizienten sowie einer Verringerung der Varianz 
 durch die Ersetzung. 
 Die Multiple Imputation basiert darauf, die fehlenden Werte durch mehrere 
 plausible Werte zu ersetzen, wobei es diverse Ersetzungsstrategien gibt. 
 Fr jeden dieser Datenstze kann dann die gewnschte Auswertung durchgefhrt 
 und anschlieend die Ergebnisse zusammengefasst werden, meist durch 
 Mittelwertbildung. Dabei sind dann die Schtzer fr die Varianz zu korrigieren 
 [little1987, rubin1987, yuan2000]. Rubin gibt an, dass eine Anzahl von 3-10 
 Imputationen ausreicht, abhngig von der Anzahl fehlender Werte [rubin1987]. 
 Fr praktische Belange reichen in der Regel 5 Imputationen. Notwendig fr die 
 Multiple Imputation ist einmal die Annahme, dass die fehlenden Werte MAR sind. 
 Zweitens ist es fr die Ersetzung notwendig, entsprechende Verteilungen  
 anzunehmen. blicherweise wird die multivariate Normalverteilung herangezogen. 
 An diesen Annahmen macht sich die Kritik fest. Vach [vach1994] und Allison 
 [allison2000] zeigen die Gefahr eines recht groen Bias bei Verletzung dieser 
 Annahmen auf und warnen entsprechend vor einem unkritischen Gebrauch dieser 
 Methode. Vach verweist allerdings auf die Mglichkeit, diese Ergebnisse im 
 Sinne einer Sensitivittsanalyse zu nutzen.

 Notwendigkeit der Deskription fehlender Werte:

 Bevor man sich jedoch fr die Anwendung von Ersetzungsstrategien fr missing
 values entscheidet, sollten davor diverse Prfungen durchgefhrt werden, um
 eine korrekte und sinnvolle Ersetzung zu gewhrleisten bzw. um deren
 Notwendigkeit zu untersuchen, denn nur mit Bedacht angewandte
 Ersetzungsstrategien liefern gute Ergebnisse bei der Analyse.

 Wie bereits im vorherigen Absatz erwhnt, sollten in einem ersten Schritt 
 die Grnde fr das Fehlen von Werten analysiert werden. Die fehlenden Werte
 sollten speziell bei der Anwedung von multiplen Imputation MAR oder MCAR sein. 
 Danach sollte die Anzahl der Variablen im Datensatz berprft werden. 
 Multiple Imputationen innerhalb des Makros "MISSING.MAC.SAS" werden mit der
 experimentellen Version von PROC MI durchgefhrt, welche bei vielen (ca. >50)
 Variablen und Beobachtungen (ca. >10000) des fteren Probleme bereitet und 
 deshalb keine korrekten Imputationen durchfhrt [lanning2003]. Laut SAS wird 
 dieses Problem erst mit dem Erscheinen der SAS Version 9 gelst sein. 

 Ferner sollte danach der Anteil von Beobachtungen mit missing values unter- 
 sucht werden. Diese Informationen erhlt man mit Hilfe des Makros
 "MISSDESCRIPTION.MAC.SAS". Ist dieser sehr niedrig, so kann erwartet werden, 
 dass sich die Complete Case-Analyse nur unwesentlich von der Analyse mit 
 Imputation unterscheiden wird, weshalb eine Imputation nicht unbedingt als 
 ntig erschient. Ist der Anteil von Beobachtungen mit missing values sehr hoch,
 so sollten in einem ersten Schritt etwaige Variablen mit sehr vielen missing 
 values identifiziert werden und evtl. aus der Analyse ausgeschlossen werden. 
 Besitzt eine Variable mehr als 50% missing values, so liefert PROC MI relativ
 schlechte Ergebnisse [qin2003].

 Innerhalb des "MISSING.MAC.SAS"-Makros wird fr multiple Imputationen die
 Prozedur MI mit der Standardmethode MCMC verwendet. Diese setzt die multivariate
 Normalverteilung voraus. Eine grobe berprfung der stetigen Variablen auf 
 Normalverteilung kann mit Hilfe des Makros "MISSDESCRIPTION.MAC.SAS" (Grafiken)
 vorgenommen werden. Liegt keine Normalverteilung vor, so kann die entsprechende
 Variable zur Imputation vorher mittels einer geeigneten Transformation in die
 Normalverteilung berfhrt werden. Diese Transformation mu nach der Imputation
 wieder rckgngig gemacht werden. Laut Shafer [shafer1999 / shafer1998] ist die 
 Imputation bei Abweichung von der multivariaten Normalverteilung relativ robust.
 Arbeitet man beispielsweise mit binren oder ordinalskalierten kategoriellen 
 Variablen, so ist es laut Shafer gerechtfertigt, deren Imputation unter Annahme 
 der Normalverteilung durchzufhren und danach auf die nchstliegende Kategorie 
 zu runden. 

 Die Handhabung der multiplen Imputation von binren oder kategoriellen Variablen
 mittels PROC MI wird in der Literatur allerdings sehr unterschiedlich beschrieben. 
 Ake [ake2002] macht den Vorschlag, eine Dummy-Kodierung fr kategorielle 
 Variablen vor der multiplen Imputation mittels PROC MI durchzufhren. 
 PROC MI ersetzt anschliessend Missing values durch stetige Werte im
 Intervall [0,1], welche danach entsprechend auf 0 oder 1 gerundet werden. 
 Nach der Imputation mssen die dummy-kodierten kategoriellen Variablen wieder
 in Ihre ursprngliche Form zurcktransformiert werden. 
 Canchola [canchola2002] ist beispielsweise der Meinung, dass PROC MI nicht in der 
 Lage ist, kategorielle Variablen zu handhaben. Diese Theorie wird durch die 
 Tatsache gesttzt, dass erst mit der SAS- Version 9 ein experimentelles CLASS- 
 Statement fr PROC MI erhltlich sein wird (SAS Version 9 Enhancements to 
 SAS/STAT Software).
 Innerhalb des Makros "MISSING.MAC.SAS" wurde die multiple Imputation mittels
 PROC MI fr kategorielle und binre Variablen so gelst, dass missing values
 stets durch stetige Werte ersetzt, und die diese danach gerundet werden. 
 Besitzt eine kategorielle Variable x beispielsweise die
 Ausprgungen 1, 5, 7 und 9, so erfolgt vor dem Aufruf von PROC MI eine Umkodierung
 in die Ausprgungen 1,2,3 und 4. PROC MI ersetzt fehlende Werte dieser Variablen
 anschliessend durch stetige Werte im Intervall [1,4] und rundet diese auf die 
 entsprechende am nchsten liegende Kategorie. Nach der Imputation werden alle
 Werte wieder in Ihre ursprnglichen Kategorien rcktransformiert. 

 Ein weiterer Punkt, der im Hinblick auf die multiple Imputation zu beachten ist,
 handelt vom sogenannten "Imputation Model". Shafer [shafer1997] gibt den Hinweis,
 dass das Imputation- Modell mit dem spteren Analyse- Modell mglichst identisch
 sein sollte. Das heit, dass nur die Variablen zur multiplen Imputation 
 herangezogen werden sollten, die auch spter fr die Analyse verwendet werden.
 
 Eine weitere Analyse, die vor der multiplen Imputation stehen sollte, ist die
 Prfung von Variablen auf Multikollinearitt, da PROC MI (MCMC-Methode) nicht in 
 der Lage ist, missing values von stark abhngigen Variablen zu ersetzen. Diese 
 Prfung kann mit Hilfe einer Multikollinearittsanalyse durchgefhrt werden.

 Des weiteren ist es im Rahmen der multiplen Imputation (PROC MI, MCMC- Methode) 
 nicht mglich, Missing values einer Variable zu erstzen, die ausschliesslich 
 fehlende Werte enthlt. Eine diesbezgliche Prfung ist innerhalb des Makros
 "MISSING.MAC.SAS" implementiert. Ebenfalls nicht mglich ist die multiple
 Imputation fehlender Werte von Variablen, die lediglich eine Ausprgung 
 besitzen (Beispiel: Variable geschlecht (0=m/1=w) enthlt lediglich die 
 Ausprgung 1 sowie einige missing values). Auch hierfr wurde eine 
 dementsprechende Prfung implementiert.

 Eine weitere Analyse, die vor etwaigen missing value- Imputationen durchgefhrt
 werden sollte, ist die Untersuchung der sogenannten 'missing value- patterns'.
 Diese bestimmen im wesentlichen die Art der multiplen Imputationsmethode,
 welche am besten verwendet werden sollte. Man unterscheidet zwei verscheidene
 Arten von missing value- patterns: Monotone missing pattern und arbitrary
 missing pattern.
 Man spricht von einem 'arbitrary missing pattern', wenn missing values
 berall auftreten knnen, also zufllig sind. Die Anordnung von Variablen
 ist in diesem Fall nicht von Bedeutung.

 Bsp.:

 Covariate Pattern   Y1   Y2   Y3
 -----------------   --   --   --
 		1			 X	  X    X
 		2            X    X    .
 		3            X    .    X
 		4            X    .    .
 		5            .    X    X
 		6            .    X    .
 		7            .    .    X
 		8            .    .    .

 Ein sogenanntes 'monotone missing pattern' liegt vor, wenn die Anordung von
 Variablen von Bedeutung ist. Ist beispielsweise die Variable Yi missing, so
 sind auch die Variablen Yi+1,... Yn missing.

 Bsp.:

 Covariate Pattern   Y1   Y2   Y3
 -----------------   --   --   --
 		1			 X	  X    X
 		2            X    X    .
 		3            X    .    .

 Eine Analyse des missing data patterns eines Datensatzes kann mit Hilfe des 
 Makros "MISSING.MAC.SAS" durchgefhrt werden. 
 Rubin [rubin1987] empfiehlt beim Vorliegen eines 'monotone missing patterns' 
 die Verwendung von parametrischen oder nichtparametrischen Regressionsmethoden 
 zur multiplen Imputation. Liegt ein zuflliges missing pattern vor, so
 verwendet man am besten die MCMC- Methode zur multiplen Imputation
 [schafer1997]. Innerhalb des Makros "PM_MISSING.MAC.SAS" wird die folgenden
 Methoden zur Verfgung gestellt:

 EM (expectation-maximization- Algorithmus)
 MCMC (Markov Chain Monte Carlo- Methode)
 REG bzw. REGRESSION (parametrische Regressionsmethode)

 Die nichtparametrische Regressionsmethode "PROPENSITY" wird nicht angeboten,
 da im Zusammenhang mit dieser Methode keine Rundungen, sowie keine Maximum- 
 und Minimumangaben mglich sind. 

 DESCRIPTION
 -----------
 Im angegeben SAS-Makro sind die beschriebenen Methoden des Umgangs mit 
 fehlenden Werten folgendermaen implementiert worden:

 - Single Imputation:  Fr stetige Variablen kann unter Nutzung der SAS-
 				 Prozedur PROC STDIZE eine Ersetzung durch den Median, 
 				 Mittelwert oder Midrange (Min + (Max-Min)/2) der vorhandenen 
 				 Beobachtungen durchgefhrt werden. Bei kategoriellen 
 				 Variablen ist die Erzeugung einer eigenen Missing-Kategorie 
 				 mglich. Der so erzeugte Datensatz kann dann zur weiteren 
 				 Bearbeitung separat abgespeichert werden. 
 - Multiple Imputation: Die Multiple Imputation wird mit der Prozedur PROC MI 
                 (ab SAS 8.2) durchgefhrt. Dabei wird im wesentlichen auf die 
                 Standardeinstellungen der Prozedur (MCMC-Methode) zurckge-
                 griffen (s. obige detaillierte Beschreibung). Zur Steuerung 
                 des Verfahrens kann die Anzahl der Imputationen 
 				 und die Rundung der stetigen Variablen im Makro angegeben 
 				 werden. Die Datenstze knnen zur weiteren Bearbeitung 
 				 abgespeichert werden.  


 LITERATURE
 ----------
 - SAS Online Doc zu den Prozeduren PROC STDIZE, PROC MI
 - Ake F, Carpenter AL (2002): Survival Analysis with PHREG:
                Using MI and MIANALYZE to Accommodate Missing Data,
   				SD VA Healthcare System, San Diego, CA; 
				Data Explorations, Carlsbad, CA
				www.wuss.org/Conference/papers/DA06.pdf 
 - Allison PD (2001): Missing data. SAGE Publications, Beverly Hills
 - Canchola A, Neilands TB, Catania JA (2002): Imputation Strategies for
				Sexual Orientation using SAS PROC MI, University of
				California, San Francisco
		http://www.hsru.org/download/wuss2002/WUSS10_Imputation_Canchola.pdf
 - Harrell FE Jr (2001): Regression modeling strategies. Springer, New York
 - Lanning D, Berry D (2003): An Alternative to PROC MI for large samples
				http://www2.sas.com/proceedings/sugi28/271-28.pdf 
 - Little RJA, Rubin DB (1987): Statistical analysis with missing data. 
 				J. Wiley & Sons, New York
 - Qin JH, Singleton M (2003): Performing Sensitivity Analyses of Imputed
				Missing Values, Kentucky CODES, Kentucky Injury Prevention
                & Research Center, University of Kentucky
				www.atsip.org/forum2003/Sessions/s06/Session6Qin.ppt 
 - Rubin DB (1987): Multiple imputation for nonresponse in surveys.  
				J. Wiley & Sons, New York
 - Scheffer J (2002): Dealing with missing data. 
				Res. Lett. Inf. Math. Sci. 3: 153-160
 - Shafer JL (1997): Analysis of incomplete multivariate data. 
				Chapman & Hall, London
 - Shafer JL (1999): Multiple Imputation: a primer, Statistical Methods in
				Medical Research, in press
 - Shafer JL, Olsen MK (1998): Multiple Imputation for multivariate 
				missing-data problems: a data analyst's perspective. 
				Multivariate Behavioral Research,33,545-571
 - Vach W (1994): Logistic regression with missing values. Springer, New York
 - Yuan YC (2000): Multiple imputation for missing data: concepts and new 
				development. SAS Institute Inc., Cary NC  
                http://support.sas.com/rnd/app/papers/multipleimputation.pdf
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
				Prognosemodellen auf Basis der logistischen Regression. 
				Diplomarbeit, FH Ulm 


SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %MISSING ( 
   *data			= ,					Datensatz
    xvar			= ,					stetige Variable(n)  
    ovar            = ,                 ordinale (/binre) Variable(n) **ergnzt am 9.5.05 durch KB
    nvar            = ,                 nominale (/binre) Variable(n) **ergnzt am 9.5.05 durch KB
   *obs_id			= ,        			Variablenname der Beobachtungsnummern im DS
    miss_patt       = 0,                Missing Pattern **ergnzt am 9.5.05 durch KB
    imputation_art	= 0,				Auswahl der Imputationsart
	miss_cat		= 99,				Kategorie fr fehlende Werte (Single Imputation)
	replace_method 	= 1,				Ersetzungsmethode fr stetige Variablen (0/1/2)
	imputation_file	= work.simputed,	Ausgabedatei fr Single Imputation
	mi_method		= MCMC,				Auswahl der multiplen Imputationsmethode
    xvar_round		= ,					Angabe der Rundung fr stetige Variablen
    range           = 1,                Angabe Wertebereich (MIN/MAX) ja (1) - nein (0) fr kategoriale Variablen
										innerhalb von PROC MI **ergnzt am 2.6.05 durch KB 
	nimpute			= 5,				Anzahl der Multiplen Imputationen
    random			= 0,				Auswahl der Zufallszahlenerzeugungsart
	mimputation_file= work.mimputed,	Ausgabedateien fr Multiple Imputation
   *macro_path		= 					Pfad zur Makrosammlung
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:			Zur Angabe des zu analysierenden Datensatzes. 
					Wurde beispielsweise vor dem Aufruf des Makros ein Datensatz mit
					dem Namen "dat300" angelegt, lautet der korrekte Aufruf: 
					data=dat300. Zustzlich kann auch die dazugehrige SAS- Library
					bergeben werden (Beispiel: data=work.dat300). 
					Die bergabe dieses Parameters ist zwingend erforderlich.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen.
					Werden mehrere stetige oder quasi- stetige Variablen bergeben, 
					so sollten diese durch ein Blank (Leerzeichen) getrennt werden. 
					Beispiel: xvar = alter bmi.

   ovar:            "ovar" steht fr "ordinal variables" und ist fr die bergabe aller
                    ordinal bzw. binr kategorialen Variablen verantwortlich. Werden
                    mehrere Variablen bergeben, sollten diese getrennt durch ein Leerzeichen
					bergeben werden.
                    **ergnzt am 18.5.05 durch KB

   nvar:            "nvar" steht fr "nominal variables" und ist fr die bergabe aller
                    nominal bzw. binr kategorialen Variablen verantwortlich. Werden
                    mehrere Variablen bergeben, sollten auch diese getrennt durch
                    ein Leerzeichen bergeben werden.
                    **ergnzt am 18.5.05 durch KB

   obs_id:			Dieser zwingend erforderliche Parameter dient der bergabe des
   					Variablennamens der Beobachtungsnummern im angegebenen Datensatz.
   					Diese Variable wird in spteren Makros bentigt, um Beobachtungen 
  					eindeutig zu identifizieren. Fr diesen Parameter wird keine 
					Missing-Value Ersetzung durchgefhrt.

   miss_patt        Der Parameter "miss_patt" dient zur Angabe des Missing Patterns
                    des angebenen Datensatzes. Um diesen Parameter richtig angeben zu
                    knnen, empfiehlt es sich vor der Ersetzung das Missing Pattern
                    durch das Makro "missing.mac.sas9.1.sas" unter Angabe "imputation_art=0"
                    ausgeben zu lassen. Wird womglich ein flschlicherweise angenommenes
                    Missing Pattern angegeben, ist es mglich, dass keine Ersetzung
                    stattfindet:
                        0=beliebiges MP     1=monotones MP.     Standard=0 
                    **ergnzt am 18.5.05 durch KB

   imputation_art:	Dieser Parameter dient der Auswahl der Imputationsart, die 
 					durchgefhrt werden sollte. 

					0: Ausgabe der Missing Value- Patterns (Standard) aus PROC MI (Keine Imputation)
					1: Single Imputation
					2: Multiple Imputation (Standard)
					3: Single & Multiple Imputation

   miss_cat:		Der Parameter 'miss_cat' steht fr "missing category" und hat
					Einfluss auf die bergebenen kategoriellen Variablen bei einer
                    Single Imputation. Hier kann diejenige Kategorie angegeben werden,
                    die die fehlenden Werte reprsentieren soll. Falls diese im Datensatz
                    bereits existieren sollte, wird die bergebene Zahl automatisch
                    erhht und die nchste freie Kategorie ausgewhlt. Standard=99.
                    Besitzt beispielsweise die unter 'ovar' bzw. 'nvar' bergebene
                    Variable x bereits die Kategorie 99 und 100, so wird die Zahl 101
                    als Kategorie, die die fehlenden Werte reprsentieren soll,
                    herangezogen. Ein entsprechender Hinweis findet sich danach im
                    OUTPUT- Fenster wieder.
                    **gendert am 18.5.05 durch KB

   replace_method:	Parameter zur Angabe der Ersetzungsmethode fr stetige Variablen bei
					Single Imputation: 1=MEDIAN, 2=MEAN, 3=MIDRANGE. Standard=1

   imputation_file:	Parameter zur Angabe eines individuellen Dateinamens fr die 
					Single Imputation. Standard=work.simputed.
					Der Bibliotheksname ist optional und kann auch weggelassen 
					werden
 
   mi_method:       Dieser Parameter dient der Auswahl der multiplen Imputationsmethode
					von PROC MI. Bei einem beliebigen Missing Pattern hat man die Wahl
                    zwischen der MCMC-Methode und dem EM-Algorithmus. Bei einem monotonem
                    Missing Patter kann man zwischen der Regressionsanalyse oder dem
                    Predicted Mean Matching whlen.
                    Standardmig ist dieser Parameter auf den Wert MCMC (Markov Chain
                    Monte Carlo) gesetzt. Folgende Methoden stehen zur Verfgung:
                    beliebiges MP:
                        MCMC (Markov Chain Monte Carlo- Methode) 
					    EM (expectation-maximization- Algorithmus)
					monotones MP:
					    REG bzw. REGRESSION (parametrische Regressionsmethode)
                        REGPMM bzw. REGPREDMEANMATCH (Predictive Mean Matching)
                   **ergnzt am 18.5.05 durch KB

   xvar_round:		Dieser Parameter greift makrointern auf PROC MI zu und setzt deren
					"Round"-Option. 'xvar_round' spezifiziert die Anzahl der 
					Nachkommastellen, auf die stetige Variablen gerundet werden sollen.
					Werden stetige Variablen mittels dem Parameter 'xvar' bergeben,
					so mssen zwingend Rundungsangaben gemacht werden. Dabei mu
					die Anzahl der Rundungsangaben mit der Anzahl der stetigen Variablen
					bereinstimmen. 
					Beispiele:

				  	xvar		= alter bmi,
				  	xvar_round  = 0.000001 0.0001,

   range:           Dieser Parameter gibt an, ob innerhalb der Prozedur PROC MI die
                    Parameter MINIMUM= und MAXIMUM= fr kategoriale Variablen gesetzt
                    werden sollen oder nicht. Wenn die Optionen MINIMUM und MAXIMUM
					der Prozedur PROC MI gesetzt werden, kann es vorkommen, dass auch
					nach 100 Versuchen kein Wert zur Ersetzung innerhalb des Wertebe-
					reiches berechnet werden kann. Wenn die Wertebereiche der kategorialen
					Variablen, in denen die zu ersetzenden Werte liegen sollen, nicht
					angegeben werden, werden die ersetzten Werte, die auerhalb des
					ursprnglichen Wertebereiches liegen, durch das eigentliche
					Minimum bzw. Maximum der Variablen ersetzt.
                    Vor der Ersetzung werden die Werte der kategorialen Variablen jeweils
					in die Werte 1 bis n umkodiert. Zum Schlu werden Werte die kategorialen
					Variablen wieder in ihre eigentlichen Werte zurcktransformiert.
					/*ergnzt am 4.6.05 durch KB

   nimpute:			Parameter zur bestimmung der Anzahl von Imputationen, die 
					durchgefhrt werden sollen. Standard=5. Wird beispielsweise
					der Wert 10 bergeben, werden 10 Datenstze mittels multipler
					Imputation (von PROC MI) erzeugt. 

   random:			Der Parameter 'random' erlaubt eine Auswahl, wie die Pseudo- 
					Zufallszahlen innerhalb von PROC MI erzeugt werden sollen. 
					Der default- Wert ist 'random=0'.

					0:	
					Hierbei wird das Systemdatum und die aktuelle Uhrzeit 
					verwendet, was zur Folge hat, dass Zufallszahlen bei jedem
					Schleifendurchlauf und bei jeder Wiederholung immer 
					unterschiedlich sind.

					positive Intergerzahl:	
					Erzeugt gleiche Ergebnisse (Ergebniskontrolle mglich)

   mimputation_file:Parameter zur Angabe der Dateinamen fr die Multiple Imputation. 
				 	Standard=work.mimputed. Wird dem Parameter 'nimpute' beispielsweise
					der Wert 5 und dem Parameter 'nimputation_file' der Wert 
					'mydat.multiple_imputed' bergeben, so werden 5 Datenstze mit den 
					Namen 'mydat.multiple_imputed1 - mydat.multiple_imputed5' erzeugt.
					Der Bibliotheksname ist optional und kann auch weggelassen 
					werden.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO MISSING ( 		data			= ,
						xvar			= ,
                        ovar            = ,  
                        nvar            = ,  
						obs_id			= ,
                        miss_patt       = 0,
						imputation_art	= 0,
						miss_cat		= 99,
						replace_method  = 1,
						imputation_file	= work.simputed,
						mi_method		= MCMC,
						xvar_round		= ,
                        range           = 1,
						nimpute			= 5,
						random			= 0,
						mimputation_file= work.mimputed,
						macro_path		= 	
			      );

/*Prfungen / Referenzierungen / Standardbelegungen*/

/***Parameter nomiss_covar auf 0 setzen, d.h es wurde keine vollstndige stetige
    Variable als ersetes bergeben*/

%LET nomiss_covar=0;


/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;
  %WINDOW error color=grey                                                       
    #5  @28 'Fehler!' attr=highlight color=blue
    #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
    #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';           
  %DISPLAY error;

  /*Sprungmarke im Fehlerfall*/
    
  %GOTO ende;
%END;


/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter
  Pfad bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);
%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
  %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
  %IF %bquote(&char1) ne %STR(%') %THEN
    %LET command='NOK';
  %ELSE
    %LET command='OK';
    %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
  %IF %bquote(&char2) ne %STR(%') %THEN
    %LET command2='NOK';
  %ELSE
    %LET command2='OK';
%END;

  /*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

  %IF &command='NOK' OR &command2='NOK' %THEN %DO;
    %WINDOW error color=grey                                                     
      #5  @28 'Fehler!' attr=highlight color=blue
	  #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert"
	  #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
	  #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
	%DISPLAY error;

  /*Sprungmarke im Fehlerfall*/
    
    %GOTO ende;
  %END;


/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;
%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
  %PUT Directory does not exist;
%END;
%ELSE %DO;
  %PUT Directory exists;
  %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;
  %WINDOW error color=grey                  
    #5  @28 'Fehler!' attr=highlight color=blue
    #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert" 
    #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"	 
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';            
  %DISPLAY error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;


/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &imputation_file= %THEN %DO;
  %LET imputation_file=simputed;
%END;

%IF &mimputation_file= %THEN %DO;
  %LET mimputation_file=mimputed;
%END;

%IF &nimpute= %THEN %DO;
  %LET nimpute=5;
%END;

%IF &miss_cat= %THEN %DO;
  %LET miss_cat=99;
%END;

%IF &replace_method= %THEN %DO;
  %LET replace_method=1;
%END;

%IF &imputation_art= %THEN %DO;
  %LET imputation_art=0;
%END;

%IF &random= %THEN %DO;
  %LET random=0;
%END;

%IF &mi_method= %THEN %DO;
  %LET mi_method=MCMC;
%END;

/***ergnzt am 18.5.05 durch KB*/
%IF &miss_patt= %THEN %DO;
  %LET miss_patt=0;
%END;   

/***ergnzt am 2.6.05 durch KB*/
%IF &range= %THEN %DO;
  %LET range=1;
%END;   


/*Externe Makros, die innerhalb des Makros 'PM_MISSING.MAC.SAS' verwendet werden
  - (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;
%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('numchar.mac.sas');


/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;


/*Test ob Parameter 'imputation_art' in {1,2,3} ist*/

%IF &imputation_art ne 0 %THEN %DO;
  %IF &imputation_art ne 1 %THEN %DO;
    %IF &imputation_art ne 2 %THEN %DO;
      %IF &imputation_art ne 3 %THEN %DO;

  /*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Parameter 'imputation_art'  <> 0 / 1 / 2 / 3!"                                                                                        
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
      %END;
    %END;
  %END;
%END;


/*berprfung, ob Datenquelle (Parameter 'data=') angegeben wurde*/

%IF &data=  %THEN %DO;

  /*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Dem Parameter 'data' (Datensatz)"  
    #8  @28 "wurde kein Wert bergeben!"   
    #9  @28 "Dieser Parameter ist jedoch zwingend erforderlich."  
    #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;


/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'ovar=' / 'nvar=')
  angegeben wurden*/ /***gendert am 18.5.05 durch KB*/

%IF &xvar= AND &ovar= AND &nvar= %THEN %DO;

  /*Falls keine Variablen angegeben wurden,*/
  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Den Parametern 'xvar', 'ovar' und 'nvar' wurden keine Variablen"
    #8  @28 "bergeben!"
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;


/*berprfung, ob dem zwingend erforderlichen Parameter 'obs_id' ein Wert bergeben
  wurde*/

%IF &obs_id= %THEN %DO;

  /*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Dem zwingend erforderlichen Parameter 'obs_id' wurde kein Wert" 
    #8  @28 "bergeben! Dieser bentigt den Variablenname der Beobachtungs-"
    #9  @28 "nummern in dem zu untersuchenden Datensatz"
    #13 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;


/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert
  (mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

  /*falls Datensatz nicht existiert, Programm beenden*/

  %IF &exist='NO' %THEN %DO;

    /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

    %window error color=grey                                                                                                                
      #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
      #7  @28 "Der unter 'data' angegebene Datensatz existiert nicht!"                                                                                        
      #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
    %display error;                                                                                                                         
                
    /*Sprungmarke im Fehlerfall*/

    %GOTO ende;
%END;


/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz
  (Parameter 'data') vorhanden sind*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/
/***gendert am 18.5.05 durch KB*/

%LET all_variables= &xvar &ovar &nvar &obs_id;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

  /*falls irgendwelche Variablen nicht im angegebenen Datensatz existieren,
    Programm beenden*/

  %IF &exist='NO' %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Eine oder mehrere bergebene Variable(n)"                                                                                        
    #8  @28 "konnte nicht im angegebenen Datensatz gefunden werden"
    #9  @28 "oder es wurde eine oder mehrere Variable(n) doppelt angegeben!"
    #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;


/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die
  bergebenen Variablen enthlt*/ /*gendert am 18.5.05*/

DATA test3;
  SET &data;
  KEEP &xvar &ovar &nvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt
  werden*/
/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,da nur diese von
  Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros
  'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
  SET testcontent;
  IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Der unter 'data' angegebene Datensatz enthlt Variablen" 
    #8  @28 "vom Typ 'character'. Diese mssen jedoch zwingend"	
    #9  @28 "vom Typ 'numeric' sein!"	  
    #13 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
  DELETE test4 testcontent test3;
RUN;
QUIT;

/***Prfen, ob insgesamt mindestens 2 Variablen zur Ersetzung bergeben wurden*/
/***ergnzt am 30.5.05*/

/*%IF &imputation_art ne 0 %THEN %DO;*/
  %IF &imputation_art ne 1 %THEN %DO;
    %IF &xvar ne OR &ovar ne OR &nvar ne %THEN %DO;
      %LET allvar=&xvar &ovar &nvar;

      /*Anzahl der bergebenen Variablen (&all_var) ermitteln*/

      %GLOBAL all_count all_var;
      %LET all_count=1;
      %LET all_var=%QSCAN(&allvar,&all_count,%STR( ));

      %DO %WHILE(&all_var ne);
        %LET all_count=%EVAL(&all_count+1);
        %LET all_var=%QSCAN(&allvar,&all_count,%STR( ));
      %END;
        %LET all_count=%EVAL(&all_count-1);

      /*Anzahl zum berblick im LOG- Fenster ausgeben*/

      %PUT The allvar contains &all_count variables.;

      %IF &all_count < 2 %THEN %DO;

        /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

        %window error color=grey                                                                                                                
          #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
          #7  @28 "Es mssen insgesamt mindestens 2 Variablen" 
          #8  @28 "zur Multiplen Ersetzung bergeben werden"	
          #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
        %display error;

        /*Sprungmarke im Fehlerfall*/

        %GOTO ende;
      %END;
    %END;
  %END;
/*%END;*/

/*Prfen, ob Variablen bergeben wurden, die ausschlielich Beobachtungen
  mit Missing Values enthlt*/

/*xvar*/

%IF &xvar ne %THEN %DO;

  /*Anzahl der angegebenen stetigen Variablen ermitteln:*/
  %GLOBAL x_count x_var;
  %LET x_count=1;
  %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));

  %DO %WHILE(&x_var ne);
    %LET x_count=%EVAL(&x_count+1);
    %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
  %END;
    %LET x_count=%EVAL(&x_count-1);

  /*Anzahl zum berblick im LOG- Fenster ausgeben*/

  %PUT The xvar contains &x_count variables.;

  %DO uhg=1 %TO &x_count;       
    %LET xvar_liste=%SCAN(&xvar,&uhg,%STR( ));

    DATA misscheck_x1 (KEEP=&xvar_liste);
      SET &data;
    RUN;

    /*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
    /*'numberobs.mac.sas' bestimmen*/
    
    %NUMBEROBS(datensatz=misscheck_x1);

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET misscheck_x1=&nnnn1;

    /*Zwischendatensatz anlegen, der nur Beobachtungen mit missing values enthlt*/

    DATA misscheck_x2 (KEEP=&xvar_liste);
      SET &data;
      IF &xvar_liste = .;
    RUN;

    /*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
    /*'numberobs.mac.sas' bestimmen*/

    %NUMBEROBS(datensatz=misscheck_x2);

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET misscheck_x2=&nnnn1;

    /*temporr angelegte Zwischendateien lschen*/

    PROC DATASETS NOLIST;
      DELETE misscheck_x2 misscheck_x1;
    RUN;
    QUIT;

    %IF &misscheck_x2 = &misscheck_x1 %THEN %DO;

      /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

      %window error color=grey                                                                                                                
        #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
        #7  @28 "Eine oder mehrere bergebene Variablen enthalten" 
        #8  @28 "ausschlielich Beobachtungen mit missing values!"	
        #9  @28 "Single/Multiple- Imputation ist deshalb nicht"	  
        #10 @28 "mglich/sinnvoll."	 
        #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
      %display error;

      /*Sprungmarke im Fehlerfall*/

      %GOTO ende;
    %END;
  %END;
%END;


/*cvar*/
/*'ovar' und 'nvar' in 'cvar' zusammenfassen - ergnzt am 25.5.05*/

%LET cvar=&ovar &nvar;

%IF &cvar ne %THEN %DO;

  /*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

  %GLOBAL c_count c_var;
  %LET c_count=1;
  %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));

  %DO %WHILE(&c_var ne);
    %LET c_count=%EVAL(&c_count+1);
    %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
  %END;
    %LET c_count=%EVAL(&c_count-1);

  /*Anzahl zum berblick im LOG- Fenster ausgeben*/

  %PUT The cvar contains &c_count variables.;

  %DO uhf=1 %TO &c_count;
    %LET cvar_liste=%SCAN(&cvar,&uhf,%STR( ));

    DATA misscheck_c1 (KEEP=&cvar_liste);
      SET &data;
    RUN;

    /*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros
      'numberobs.mac.sas' bestimmen*/

    %NUMBEROBS(datensatz=misscheck_c1);

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET misscheck_c1=&nnnn1;

    /*Zwischendatensatz anlegen, der nur Beobachtungen mit missing values enthlt*/

    DATA misscheck_c2 (KEEP=&cvar_liste);
      SET &data;
      IF &cvar_liste = .;
    RUN;

    /*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros
      'numberobs.mac.sas' bestimmen*/

    %NUMBEROBS(datensatz=misscheck_c2);

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET misscheck_c2=&nnnn1;

    /*temporr angelegte Zwischendateien lschen*/

    PROC DATASETS NOLIST;
      DELETE misscheck_c2 misscheck_c1;
    RUN;
    QUIT;

    %IF &misscheck_c2 = &misscheck_c1 %THEN %DO;

      /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

      %window error color=grey                                                                                                                
        #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
        #7  @28 "Eine oder mehrere bergebene Variablen enthalten" 
        #8  @28 "ausschlielich Beobachtungen mit missing values!"	
        #9  @28 "Single/Multiple- Imputation ist deshalb nicht"	  
        #10 @28 "mglich/sinnvoll."	 
        #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
      %display error;

      /*Sprungmarke im Fehlerfall*/

      %GOTO ende;
    %END;
  %END;
%END;


/*Test, ob bei bestimmten Parametern anstatt numerischen Werten character- Werte
  bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/
/***ergnzt am 18.5.05 durch KB*/

%NUMCHAR(parameterliste=&xvar_round &nimpute &miss_cat &replace_method
         &imputation_art &random &miss_patt);

%IF &numcharok='NOK' %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Einem oder mehreren Parameter(n) wurde(n) anstatt eines" 
    #8  @28 "numerischen Wertes flschlicherweise ein alphanumerischer Wert bergeben!"	 
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;


/*berprfung, ob Libnames der Zieldateien vorhanden sind falls welche
  angegeben wurden: falls 'imputation_art=1 oder =3' (Single Imputation)*/

%IF &imputation_art ne 0 %THEN %DO;
  %IF &imputation_art ne 2 %THEN %DO;
    %IF &imputation_file ne %THEN %DO;

  /*Prfung ob Dateinamen mit Libname bergeben wurde*/

  %LET length=%LENGTH(&imputation_file);
  %LET point='NO';

  %DO h=1 %TO &length;
    %LET check=%SUBSTR(&imputation_file,&h,1);
  	%IF &check=. %THEN %DO;
      %LET point='YES';
	%END;
  %END;

  %IF &point='YES' %THEN %DO;
    %LET lib=%QSCAN(&imputation_file,1,.);
    %LET file=%QSCAN(&imputation_file,2,.);

    DATA &lib..test;
      test='Test';
    RUN;

    /*berprfen, ob das angegebene Zielverzeichnis auch wirklich existiert
      (mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

    %EXIST(dsn=&lib..test);

    /*temporre Zwischendateien lschen*/

    PROC DATASETS NOLIST LIBRARY=&lib;
	  DELETE test;
    RUN;
    QUIT;
  %END;

  %IF &exist ='NO' %THEN %DO;

    /*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

    %window error color=grey                                                                                                                
      #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
      #7  @28 "Libname fr eine oder mehrere Ausgabedateien" 
	  #8  @28 "existiert nicht! Bitte berprfen."	 
      #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
    %display error;

    /*Sprungmarke im Fehlerfall*/

    %GOTO ende;
  %END;
     %END;
  %END;
%END;


/*berprfung, ob Libnames der Zieldateien vorhanden sind falls welche
  angegeben wurden: falls 'imputation_art=2 oder =3' (Multiple Imputation)*/

%IF &imputation_art ne 1 %THEN %DO;
  %IF &mimputation_file ne %THEN %DO;

  /*Prfung ob Dateinamen mit Libname bergeben wurde*/

  %LET length=%LENGTH(&mimputation_file);
  %LET point='NO';

  %DO h=1 %TO &length;
    %LET check=%SUBSTR(&mimputation_file,&h,1);
	%IF &check=. %THEN %DO;
  	  %LET point='YES';
	%END;
  %END;

  %IF &point='YES' %THEN %DO;
    %LET lib=%QSCAN(&mimputation_file,1,.);
    %LET file=%QSCAN(&mimputation_file,2,.);

    DATA &lib..test;
	  test='Test';
    RUN;

    /*berprfen, ob das angegebene Zielverzeichnis auch wirklich existiert
      (mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

    %EXIST(dsn=&lib..test);

    PROC DATASETS NOLIST LIBRARY=&lib;
	  DELETE test;
    RUN;
    QUIT;

    %IF &exist ='NO' %THEN %DO;

    /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

    %window error color=grey                                                                                                                
      #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
      #7  @28 "Libname fr eine oder mehrere Ausgabedateien" 
      #8  @28 "existiert nicht! Bitte berprfen."	 
      #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
    %display error;

    /*Sprungmarke im Fehlerfall*/

    %GOTO ende;
  %END;
%END;
  %END;
%END;


/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &nimpute <=0 %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Einem oder mehreren Parameter(n) wurde(n) anstatt eines" 
    #8  @28 "positiven Wertes flschlicherweise ein negativer Wert oder 0 bergeben!"	 
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/
    
  %GOTO ende;
%END;


/*Prfen ob Parameter 'mi_method' MCMC, EM, REG/REGRESSION oder REGPMM/
  REGPREDMEANMATCH bergeben wurde*/ /***ergnzt am 18.5.05 durch KB*/

%IF %UPCASE(&mi_method) ne MCMC %THEN %DO;
  %IF %UPCASE(&mi_method) ne REG %THEN %DO;
    %IF %UPCASE(&mi_method) ne REGRESSION %THEN %DO;
      %IF %UPCASE(&mi_method) ne EM %THEN %DO;
        %IF %UPCASE(&mi_method) ne REGPMM %THEN %DO;
          %IF %UPCASE(&mi_method) ne REGPREDMEANMATCH %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/
    
  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Dem Parameter 'mi_method' wurde ein syntaktisch" 
    #8  @28 "falscher Wert bergeben! Mgliche Angaben:" 
    #9  @28 "MCMC | REG | REGRESSION | EM | REGPMM | REGPREDMEANMATCH" 
    #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
          %END;
        %END;
      %END;
    %END;
  %END;
%END;


/*Single Imputation*/

%IF &imputation_art ne 0 %THEN %DO;
  %IF &imputation_art ne 2 %THEN %DO;

/*Test ob Parameter 'replace_method' in {1,2,3} ist*/

%IF &replace_method ne 1 %THEN %DO;
  %IF &replace_method ne 2 %THEN %DO;
    %IF &replace_method ne 3 %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Parameter 'replace_method' <> 1 / 2 / 3!"                                                                                        
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;

    %END;
  %END;
%END;

  %END;
%END;


/*Multiple Imputation*/

/*Prfung ob die # Rundungsangaben mit der # stetiger Variablen bereinstimmt,
  falls welche angegeben wurden*/

%IF &imputation_art ne 1 %THEN %DO;
  %IF &xvar ne %THEN %DO;

  /*Anzahl der angegebenen stetigen Variablen ermitteln:*/

  %GLOBAL x_count x_var;
  %LET x_count=1;
  %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));

  %DO %WHILE(&x_var ne);
    %LET x_count=%EVAL(&x_count+1);
    %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
  %END;
    %LET x_count=%EVAL(&x_count-1);

  /*Anzahl zum berblick im LOG- Fenster ausgeben*/

  %PUT The xvar contains &x_count variables.;

  /*Anzahl der Rundungen ermitteln*/

  %GLOBAL ref_count var_ref;
  %LET ref_count=1;
  %LET var_ref=%QSCAN(&xvar_round,&ref_count,%STR( ));

  %DO %WHILE(&var_ref ne);
    %LET ref_count=%EVAL(&ref_count+1);
    %LET var_ref=%QSCAN(&xvar_round,&ref_count,%STR( ));
  %END;
    %LET ref_count=%EVAL(&ref_count-1);

  /*Anzahl ausgeben im LOG- Fenster*/
    
  %PUT The &xvar_round contains &ref_count variables.;

  %IF &x_count ne &ref_count %THEN %DO;

    /*falls Anzahl der stetigen Variablen grer Null ist und nicht mit der Anzahl
      der angegebenen Referenzkategorien bereinstimmt, Anzeige eines Fensters
      mit Fehlermeldung/Hinweis*/

    %window error color=grey                                                                                                                
      #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
      #7  @28 "Die Anzahl der stetigen Variablen (Parameter 'xvar') und die Anzahl "
	  #8  @28 "der Rundungsangaben (Parameter 'xvar_round') stimmt nicht berein!"                                                                                        
      #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
    %display error;

    /*Sprungmarke im Fehlerfall*/

	%GOTO ende;
  %END;
  %END;
%END;


/***Prfen, ob Parameter 'miss_patt' in {0,1} ist*/
/***ergnzt am 18.05.05 durch KB*/

%IF &miss_patt ne 0 %THEN %DO;
  %IF &miss_patt ne 1 %THEN %DO;

  /*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Parameter 'miss_patt'  <>  0 / 1!"                                                                                        
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
  %END;
%END;


/***Prfen ob die richtige MI-Methode zun angegebenen Missing Pattern
    bergeben wurde*/ /***ergnzt am 18.05.05 durch KB*/

/***Beliebiges Missing Pattern*/
%IF &imputation_art ne 1 %THEN %DO;
  %IF &imputation_art ne 0 %THEN %DO;
    %IF &miss_patt ne 1 %THEN %DO;
      %IF %UPCASE(&mi_method) ne MCMC %THEN %DO;
        %IF %UPCASE(&mi_method) ne EM %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %WINDOW error color=grey
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Bei einem BELIEBIGEN Missing Pattern (miss_patt=0)"
    #8  @28 "ist nur die MI-Methode 'MCMC' oder 'EM' zulssig!"                                                                                        
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %DISPLAY error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
        %END;
      %END;
    %END;
  %END;
%END;

/***Monotones Missing Pattern*/
%IF &imputation_art ne 1 %THEN %DO;
  %IF &imputation_art ne 0 %THEN %DO;
    %IF &miss_patt ne 0 %THEN %DO;
      %IF %UPCASE(&mi_method) ne REG %THEN %DO;
        %IF %UPCASE(&mi_method) ne REGRESSION %THEN %DO;
          %IF %UPCASE(&mi_method) ne REGPMM %THEN %DO;
            %IF %UPCASE(&mi_method) ne REGPREDMEANMATCH %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %WINDOW error color=grey
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Bei einem MONOTONEM Missing Pattern (miss_patt=1)"
    #8  @28 "ist nur die MI-Methode 'REG/REGRESSION' oder"
    #9  @28 "'REGPMM/REGPREDMEANMATCH' zulssig!"                                                                                        
    #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %DISPLAY error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
            %END;
          %END;
        %END;
      %END;
    %END;
  %END;
%END;


/*prfen, ob Variable 'obs_id' missings enthlt - (diese mu eindeutig sein)*/

/*Erzeugung eines temporren Zwischendatensatzes, der nur diese Variable enthlt*/

DATA test;
  SET &data;
  KEEP=&obs_id;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test1=&nnnn1;

/*Elimination aller etwaiger missing values der Variable 'obs_id'*/

DATA test2;
  SET test;
  IF &obs_id=. THEN DELETE;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test2);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test2=&nnnn1;

%IF &test1 ne &test2 %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Die Identifikationsvariable der Beobachtungen, welche unter" 
	#8  @28 "dem Parameter 'obs_id' angegeben wurde enthlt missing values!"	
	#9  @28 "Beobachtungen knnen nicht eindeutig identifiziert werden."	  
    #13 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
  DELETE test test2;
RUN;
QUIT;

/*mittels PROC FREQ prfen ob Variable 'obs_id' eindeutig ist*/

/*PROC FREQ- Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnistabelle mittels ODS ermitteln*/

ODS OUTPUT OneWayFreqs=freqs;
  PROC FREQ DATA=&data;
    TABLES &obs_id;
  RUN;

/*ODS Output schlieen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*Lediglich die Information in der Spalte 'frequency' ist von Bedeutung*/

DATA freqs;
  SET freqs;
  KEEP frequency;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freqs);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET freqs2=&nnnn1;

/*jede Beobachtungsnummer darf nur 1 x vorkommen*/

DATA freqs;
  SET freqs;
  IF frequency=1;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freqs);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET freqs=&nnnn1;

%IF &freqs2 ne &freqs %THEN %DO;

  /*Anzeige eines Fensters mit entsprechender Fehlermeldung/Hinweis*/

  %window error color=grey                                                                                                                
    #5  @28 'Fehler!' attr=highlight color=blue                                                                                                                
    #7  @28 "Die unter 'obs_id' angegebene Beobachtungsidentifikationsnummer" 
	#8  @28 "ist nicht eindeutig! Dies ist jedoch zwingend erforderlich."	
	#9  @28 "Einfhrung einer neuen Identifikationsnummer: "	  
	#10 @28 "DATA mydat; SET mydat; obs_id_neu=_N_; RUN;"	 
    #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
  %display error;

  /*Sprungmarke im Fehlerfall*/

  %GOTO ende;
%END;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
  DELETE freqs;
RUN;
QUIT;


/*Prfen ob 'xvar'/ 'cvar'='ovar'&'nvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*Prfung bei multipler Imputation*/

%IF &imputation_art ne 0 %THEN %DO;
  %IF &imputation_art ne 1 %THEN %DO;

/*xvar*/

%IF &xvar ne %THEN %DO;

  /*Anzahl der angegebenen stetigen Variablen ermitteln:*/

  %GLOBAL x_count x_var;
  %LET x_count=1;
  %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));

  %DO %WHILE(&x_var ne);
    %LET x_count=%EVAL(&x_count+1);
    %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
  %END;
    %LET x_count=%EVAL(&x_count-1);

  /*Anzahl zum berblick im LOG- Fenster ausgeben*/

  %PUT The xvar contains &x_count variables.;

  %DO uhgu=1 %TO &x_count;
    %LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

    /*Temporrer Zwischendatensatz enthlt lediglich Beobachtungen*/
    /*ohne missing values*/

    DATA freq2;
      SET &data;
      IF &xvar_liste ne .;
    RUN;

    /*Output unterdrcken*/

    ODS LISTING CLOSE;

    /*Ergebnisse mittels ODS bestimmen*/
    
    ODS OUTPUT OneWayFreqs=freq;

    PROC FREQ DATA=freq2;
      TABLES &xvar_liste;
    RUN;
    
    /*ODS Output wieder schliessen*/
    
    ODS OUTPUT CLOSE;
    ODS LISTING;

    /*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

    %NUMBEROBS(datensatz=freq);

    /*temporr angelegte Zwischendateien lschen*/

    PROC DATASETS NOLIST;
    	DELETE freq freq2;
    RUN;
    QUIT;

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET xvar_check=&nnnn1;

    %IF &xvar_check=1 %THEN %DO;
      %LET category_test=1;
      %LET category_test_fail=&category_test_fail &xvar_liste;
    %END;
  %END;
%END;

/*cvar*/
/*'ovar' und 'nvar' in 'cvar' zusammenfassen - ergnzt am 25.5.05*/

%LET cvar=&ovar &nvar;

%IF &cvar ne %THEN %DO;

  /*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

  %GLOBAL c_count c_var;
  %LET c_count=1;
  %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));

  %DO %WHILE(&c_var ne);
    %LET c_count=%EVAL(&c_count+1);
    %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
  %END;
    %LET c_count=%EVAL(&c_count-1);

  /*Anzahl zum berblick im LOG- Fenster ausgeben*/

  %PUT The cvar contains &c_count variables.;

  %DO uhfu=1 %TO &c_count;
    %LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

    /*Temporrer Zwischendatensatz enthlt lediglich Beobachtungen*/
    /*ohne missing values*/
    
    DATA freq2;
      SET &data;
      IF &cvar_liste ne .;
    RUN;
    
    /*Output unterdrcken*/

    ODS LISTING CLOSE;

    /*Ergebnisse mittels ODS bestimmen*/

    ODS OUTPUT OneWayFreqs=freq;

    PROC FREQ DATA=freq2;
      TABLES &cvar_liste;
    RUN;
    
    /*ODS Output wieder schliessen*/
    
    ODS OUTPUT CLOSE;
    ODS LISTING;
    
    /*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

    %NUMBEROBS(datensatz=freq);

    /*temporr angelegte Zwischendateien lschen*/

    PROC DATASETS NOLIST;
      DELETE freq freq2;
    RUN;
    QUIT;

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET cvar_check=&nnnn1;

    %IF &cvar_check=1 %THEN %DO;
      %LET category_test=1;
      %LET category_test_fail=&category_test_fail &cvar_liste;
    %END;
  %END;
%END;


/*Prfen, ob die erste angegebene stetige Variable vollstndig, d.h. keine
  Missing Values enthlt, ist, wenn ein monotones MP vorliegt*/

%IF &miss_patt=1 %THEN %DO;
  %IF &xvar ne %THEN %DO;
    %IF &ovar ne OR &nvar ne %THEN %DO;

  %DO uhg=1 %TO 1;       
    %LET xvar_liste=%SCAN(&xvar,&uhg,%STR( ));

    DATA misscheck_x1 (KEEP=&xvar_liste);
      SET &data;
    RUN;

    /*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
    /*'numberobs.mac.sas' bestimmen*/
    
    %NUMBEROBS(datensatz=misscheck_x1);

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET misscheck_x1=&nnnn1;

    /*Zwischendatensatz anlegen, der nur Beobachtungen mit missing values enthlt*/

    DATA misscheck_x2 (KEEP=&xvar_liste);
      SET &data;
      IF &xvar_liste = . THEN DELETE;
    RUN;

    /*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
    /*'numberobs.mac.sas' bestimmen*/

    %NUMBEROBS(datensatz=misscheck_x2);

    /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

    %LET misscheck_x2=&nnnn1;

    /*temporr angelegte Zwischendateien lschen*/

    PROC DATASETS NOLIST;
      DELETE misscheck_x2 misscheck_x1;
    RUN;
    QUIT;

    %IF &misscheck_x2 = &misscheck_x1 %THEN %DO;
      %LET nomiss_covar=1;
    %END;
  %END;
    %END;
  %END;
%END;

  %END;
%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/
/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=&data);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));

%DO %WHILE(&varx ne);
  %LET count_x=%EVAL(&count_x+1);
  %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
  %LET count_x=%EVAL(&count_x-1);


/*Anzahl der kategorialen Variablen ermitteln:*/
/*'ovar' und 'nvar' in 'cvar' zusammenfassen - ergnzt am 25.5.05*/

%LET cvar=&ovar &nvar;

%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));

%DO %WHILE(&varc ne);
  %LET count_c=%EVAL(&count_c+1);
  %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
  %LET count_c=%EVAL(&count_c-1);

/*Elimination aller missing values*/

DATA work.fallzahl;
  SET &data;

  %DO i=1 %TO &count_x;
    %LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
	IF &xvar_liste=. THEN DELETE;
  %END;

  %DO i=1 %TO &count_c;
    %LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
	IF &cvar_liste=. THEN DELETE;
  %END;
RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahldata=%SYSEVALF(&fallzahl-&nnnn1);

/*Makro-kopf ausgeben*/

DATA standardoutput;
  makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
  VAR makroinfo;

  TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
  TITLE2 ;
  TITLE3 "Allgemeine Makroinformationen:           ";
  TITLE4 ;
  TITLE5 "Makroname:              &sysmacroname    ";
  TITLE6 "Analysierter Datensatz: &datensatz       ";
  TITLE7 "Fallzahl:               &fallzahl        ";
  TITLE8 "Datum:                  &sysday &sysdate ";
  TITLE9 "Start der SAS-Session:  &systime         ";

  LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
  DELETE standardoutput fallzahl;
RUN;
QUIT;

/*ENDE Standardoutput*/

/*----------------------------------------------------------------------*/

/*zuerst, temporren Datensatz anlegen, der nur die bergebenen Variablen enthlt:*/
/***gendert am 19.05.05*/

DATA _tmp_ (KEEP= &xvar &ovar &nvar &obs_id);
  SET &data;
RUN;

DATA _tmpx_ (KEEP= &xvar &ovar &nvar &obs_id);
  SET &data;
RUN;

/*Anzahl der kategorialen Variablen ermitteln:*/
/*'ovar' und 'nvar' in 'cvar' zusammenfassen - ergnzt am 25.5.05*/

%LET cvar=&ovar &nvar;

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));

%DO %WHILE(&c_var ne);
  %LET c_count=%EVAL(&c_count+1);
  %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
  %LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der angegebenen stetigen Variablen ermitteln:*/

%GLOBAL x_count x_var;
%LET x_count=1;
%LET x_var=%QSCAN(&xvar,&x_count,%STR( ));

%DO %WHILE(&x_var ne);
  %LET x_count=%EVAL(&x_count+1);
  %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
%END;
  %LET x_count=%EVAL(&x_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The xvar contains &x_count variables.;

/*Missing- Value- Indikatoren fr alle Variablen einfhren:*/
/*0=nein, 1=ja*/

DATA _tmp_;
  SET _tmp_;

  /*stetige Variablen*/

  %DO i=1 %TO &x_count;
    %LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
	IF &xvar_liste=. THEN
      &&xvar_liste._missval=1;
	ELSE
      &&xvar_liste._missval=0;
  %END;

  /*kategorielle Variablen*/

  %DO h=1 %TO &c_count;
    %LET cvar_liste=%SCAN(&cvar,&h,%STR( ));
	IF &cvar_liste=. THEN
      &&cvar_liste._missval=1;
	ELSE
      &&cvar_liste._missval=0;
  %END;

  /*neue fortlaufende Beobachtungsnummer*/

  _newobs_=_N_;
RUN;

/*Ermitteln, wieviel Prozent der Beobachtungen missing values enthalten:*/

/*#Beobachtungen im Originaldatensatz bestimmen:*/

%NUMBEROBS(datensatz=&data);

%LET data_count =&nnnn1;

/*alle missing values entfernen*/

DATA nomiss;
  SET  &data;

  %DO i=1 %TO &x_count;
    %LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
	IF &xvar_liste=. THEN DELETE;
  %END;

  %DO i=1 %TO &c_count;
    %LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
	IF &cvar_liste=. THEN DELETE;
  %END;
RUN;

/*#Beobachtungen im Datensatz ohne missing values bestimmen:*/

%NUMBEROBS(datensatz=nomiss);

%LET miss_count =&nnnn1;

/*Prozentzahl bestimmen*/

%LET factor=100;
%LET x=%SYSEVALF(&data_count/&factor);
%LET miss=%SYSEVALF(&data_count-&miss_count);
%LET percent_miss=%SYSEVALF(&miss/&x);

/*Prozentzahl auf eine Stellen nach dem Komma runden*/

DATA round;
  percent_miss=&percent_miss;
  call symput('percent_miss',trim(left(put(percent_miss,BEST5.))));
RUN;

%PUT &percent_miss;

/*berprfen, ob berhaupt missing values im Datensatz vorhanden sind:*/

%IF &miss = 0 %THEN %DO;

  /*--> keine missing values vorhanden, keine (multiple) Imputation*/

  %GOTO nomissing;
%END;


/*****************************************************************************/
/******************************Single Imputation******************************/
/*****************************************************************************/

%IF &imputation_art ne 0 %THEN %DO;
  %IF &imputation_art ne 2 %THEN %DO;

/*falls stetige Variablen bergeben wurden:*/

%IF &xvar ne %THEN %DO;

  /*Median/Mean/Midrange- Ersetzung bei missing values von stetigen Variablen*/

  %IF &replace_method=1 %THEN %DO;
    %LET method=MEDIAN;
  %END;

  %IF &replace_method=2 %THEN %DO;
    %LET method=MEAN;
  %END;

  %IF &replace_method=3 %THEN %DO;
    %LET method=MIDRANGE;
  %END;

  /*Automatische Ersetzung von missing values durch PROC STDIZE*/
  /*xvar*/

  PROC STDIZE DATA=_tmp_ REPONLY METHOD=&method OUT=imputed;
    VAR &xvar;
  RUN;
%END;

/*kategorielle Variablen*/

/*Nullinitialisierung von Variablen*/

%GLOBAL missv;
%GLOBAL varstring;

/*Bei kategoriellen Variablen: Neue Kategorie einfhren, die missing values reprsentiert*/

%DO m=1 %TO &c_count;
  %LET var=%SCAN(&cvar,&m,%STR( ));

  /*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

  PROC FREQ DATA=_tmp_ NOPRINT;
    TABLES &var /  SPARSE OUT=&var;
  RUN;

  /*Ausgabedatensatz von PROC FREQ anpassen*/

  DATA &var (DROP=percent count);
    SET &var;

    /*falls Ausprgung <0 ohnehin nicht von Interesse*/
	IF &var <0 THEN
        DELETE;
	IF &var =. THEN
        DELETE;
  RUN;

  /*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/

  %NUMBEROBS(datensatz=&var);

  %LET count_var=&nnnn1;

  /*Ausprgungen als Makro- Variable abspeichern abspeichern*/

  DATA &var;
    SET &var;
	CALL SYMPUT('y'||left(_N_),&var);
  RUN;

  %LET missingv=&miss_cat;

  /*Sprungmarke fr den Fall, dass die missing- value Kategorie 99
    bereits in den Daten vorkommt*/

  %weiter:

  /*Prfvariable*/

  %LET check=0;

  %DO g=1 %TO &count_var;

    /*berprfung, ob Kategorie fr missing values bereits vergeben ist*/

	%IF &&y&g ne &missingv %THEN %DO;
      %LET check=%SYSEVALF(&check+1);
	%END;
  %END;

  DATA imputed;
    SET 

    /***ergnzt am 19.05.05 durch KB*/
    /***dass Ersetzung bei SI auch mglich ist, wenn nur kategoriale Variablen
        bergeben werden*/

    %IF &xvar ne OR &m > 1 %THEN %DO;
      imputed
	%END;

    %ELSE %DO;
	  _tmp_
 	%END;
    ;

    /*Falls nicht, Ersetzung aller missing values durch 99*/

	%IF &check = &count_var %THEN %DO;
	  IF &var=. THEN
        &var=&missingv;
	%END;

    %ELSE %DO;

      /*Falls ja, Erhhung der missing- value- Kategorie und erneuter Test*/

      %LET missingv=%SYSEVALF(&missingv+1);

      %GOTO weiter;
    %END;
  RUN;

  /*berflssige, temporre Zwischedateien lschen*/

  PROC DATASETS NOLIST;
    DELETE &var;
  RUN;
  QUIT;

  /*jeweilige Variable und jeweilige Referenzkategorie fr missing values
    an String anhngen*/

  %LET missv= &missv &missingv;
  %LET varstring= &varstring &var;

  /*DM "clear log;";*/

%END;

/*eingefhrte neue Beobachtungsnummer wieder lschen*/

DATA &imputation_file (DROP=_newobs_);
  SET imputed;
RUN;

/*Strings in Datensatz schreiben, damit dieser im Output Fenster ausgegeben
  werden kann*/

/*cvar*/

DATA var_uebersicht;
  variable		   ='                                                         ';
  missing_kategorie='                                                         ';
  miss_indi		   ='                                                         ';

  %DO m=1 %TO &c_count;
    %LET missing_indi=%scan(&cvar,&m," ");
	%LET miss_indi=&&missing_indi._missval;

	variable="%scan(&varstring,&m," ")";
	missing_kategorie="%scan(&missv,&m," ")";
	miss_indi="&miss_indi";
	output var_uebersicht;
  %END;
RUN;

/*xvar*/

DATA var_uebersicht2;
  variable2		 ='                                                           ';
  replace_method ='                                                           ';
  miss_indi		 ='                                                           ';

  %DO j=1 %TO &x_count;
    %LET missing_indi=%scan(&xvar,&j," ");
    %LET miss_indi=&&missing_indi._missval;

 	variable2="%scan(&xvar,&j," ")";
	replace_method="&method";
	miss_indi="&miss_indi";
	output var_uebersicht2;
  %END;
RUN;

DATA null;
	ersetz="&imputation_file";
RUN;

/*Strings zurcksetzen:*/

%LET missv=;
%LET varstring=;


/*Datenstze lschen*/

PROC DATASETS NOLIST;
  DELETE  round imputed;
RUN;
QUIT;
  %END;
%END;


/*****************************************************************************/
/*****************************Multiple Imputation*****************************/
/*****************************************************************************/

%IF &category_test=0 %THEN %DO;
  %IF &imputation_art ne 1 %THEN %DO;

/*Bestimmung der minimalen & maximalen Kategorie aller kategoriellen Variablen
 (fr MINIMUM= / MAXIMUM= Optionen bei PROC MI), wenn keine vollstndige stetige
  Variable und Range=1 (Ja) bergeben wurde*/

%IF &nomiss_covar=0 AND &range=1 %THEN %DO; /*&range=1 ergnzt am 6.6.05*/

/*'ovar' und 'nvar' in 'cvar' zusammenfassen - ergnzt am 25.5.05*/
%LET cvar=&ovar &nvar;

  /***ergnzt am 19.05.05 durch KB*/
  /***Anzahl der kategorialen Variablen ermitteln*/

  %GLOBAL c_count c_var;
  %LET c_count=1;
  %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));

  %DO %WHILE(&c_var ne);
    %LET c_count=%EVAL(&c_count+1);
    %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
  %END;
    %LET c_count=%EVAL(&c_count-1);


  /*Nullinitialisierung des Strings, der alle minimalen Kategorien enthlt*/
  /*minimale Kategorie nach Umkodierung immer 1*/

  %LET minlist=;

  /*Nullinitialisierung des Strings, der alle maximalen Kategorien enthlt*/
  /*maximale Kategorie nach Umkodierung immer # Kategorien je Variable */

  %LET maxlist=;

  /*Nullinitialisierung des Strings, der die neuen Kategoriennamen enthlt*/
  /*ist immer obs_Variablenname */

  %LET new_cvar=;

  /*Nullinitialisierung des Strings, der die "Round- Kategorien" fr PROC MI enthlt*/
  /*ist immer 1 (ganze Zahl) */

  %LET cvar_round=;

  %DO d=1 %TO &c_count;
    %LET c_var=%SCAN(&cvar,&d,%STR( ));

    /*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

    PROC FREQ DATA=_tmpx_ NOPRINT;
      TABLES &c_var /  SPARSE OUT=&c_var;
    RUN;

    /*Ausgabedatensatz von PROC FREQ anpassen*/

    DATA &c_var (DROP=percent count);
      SET &c_var;
      IF &c_var=. THEN DELETE;
    RUN;

    /*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/

    %NUMBEROBS(datensatz=&c_var);
    
    %LET count_var=&nnnn1;

    /*neue Beobachtungsnummer einfhren*/
    
    DATA &c_var ;
      SET &c_var;
      obs_&c_var=_N_;
    RUN;

    /*jeweilige minimale Kategorie an String anhngen (immer 1)*/
    
    %LET minlist=&minlist 1;

    /*jeweilige maximale Kategorie an String anhngen*/

    %IF &count_var=1 %THEN %DO;
      %LET maxlist=&maxlist .;
   	%END;

   	%ELSE %DO;
   	  %LET maxlist=&maxlist &count_var;
   	%END;

    /*jeweiliger neuer Variablenname an String anhngen*/
    
    %LET new_cvar=&new_cvar obs_&c_var;

    /*jeweilige "Round- Zahl" an String anhngen (immer 1)*/

    %LET cvar_round=&cvar_round 1;
  %END;
%END;

/*-----------------------*/

/*minimalen & maximalen Wert aller stetigen Variablen bestimmen*/
/*(fr MINIMUM= / MAXIMUM= Optionen bei PROC MI)*/

%IF &xvar ne %THEN %DO;

  /*Nullinitialisierung des Strings, der alle minimalen Werte enthlt*/

  %LET minlistx=;

  /*Nullinitialisierung des Strings, der alle maximalen Werte enthlt*/

  %LET maxlistx=;

  %DO d=1 %TO &x_count;
    %LET var=%SCAN(&xvar,&d,%STR( ));

    /*Ausprgungen der jeweiligen stetigen Variable ermitteln*/

    PROC FREQ DATA=_tmpx_ NOPRINT;
      TABLES &var /  SPARSE OUT=&var;
    RUN;

    /*Ausgabedatensatz von PROC FREQ anpassen*/

    DATA &var (DROP=percent count);
      SET &var;
      IF &var=. THEN DELETE;
    RUN;

    /*Anzahl der mglichen Ausprgungen je stetiger Variable ermitteln*/
    
   	%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

    /*neue Beobachtungsnummer einfhren*/

    DATA &var ;
      SET &var;
      obs=_N_;
      absolute=(abs(&var)*2);
    RUN;

    /*erste Kategorie fr Minimum als Makro- Variable abspeichern*/

    /*alle Beobachtungen bis auf Nr.1 lschen*/

    DATA &var&d ;
      SET &var;
      IF obs ne 1 THEN DELETE;
    RUN;

    /*erste Kategorie als Makro- Variable abspeichern*/

    DATA &var&d ;
      SET &var&d;
      CALL SYMPUT('y',&var);
      CALL SYMPUT('ya',absolute);
    RUN;

    /*jeweilige minimale Kategorie an String anhngen*/
    
	%LET minlistx=&minlistx &y;

    /*letzte Kategorie fr Maximum als Makro- Variable abspeichern*/

    /*alle Beobachtungen bis auf die letzte lschen*/

    DATA &var ;
      SET &var;
      IF obs ne &count_var THEN DELETE;
    RUN;

    /*letzte Kategorie als Makro- Variable abspeichern*/

    DATA &var;
      SET &var;
      CALL SYMPUT('x',&var);
      CALL SYMPUT('xa',absolute);
    RUN;

    /*jeweilige maximale Kategorie an String anhngen*/
    
	%LET yc=%SYSEVALF(&y+&ya+1);
	%LET xc=%SYSEVALF(&x+&xa+1);
	
	%IF &xc ne &yc %THEN %DO;
   	  %LET maxlistx=&maxlistx &x;
   	%END;

    %ELSE %DO;
   	  %LET maxlistx=&maxlistx .;
   	%END;

        
   	PROC DATASETS NOLIST;
	  DELETE &var&d &var ;
    RUN;
    QUIT;
  %END;
%END;

/*-------------------------*/

/*Erstellen eines temporren Datensatzes mit umkodierten kategorialen Variablen
  in [1,n] und stetigen Variablen, wenn keine vollstndige stetige Variable und
  Range=0 (Nein) bergeben wurde*/ /*ergnzt am 6.6.05*/

%IF &nomiss_covar=0 AND &range=0 %THEN %DO;

  /*Nullinitialisierung des Strings, der alle minimalen Kategorien enthlt*/
  /*minimale Kategorie immer .*/

  %LET minlist=;

  /*Nullinitialisierung des Strings, der alle maximalen Kategorien enthlt*/
  /*maximale Kategorie immer .*/

  %LET maxlist=;

  /*Nullinitialisierung des Strings, der die "Round- Kategorien" fr PROC MI enthlt*/
  /*ist immer 1 (ganze Zahl) */

  %LET cvar_round=;

  DATA _trans_ (KEEP=&obs_id &xvar &ovar &nvar);
    SET &data;
  RUN;

  /*'ovar' und 'nvar' in 'cvar' zusammenfassen*/
  %LET cvar=&ovar &nvar;

  /***Anzahl der kategorialen Variablen ermitteln*/

  %GLOBAL c_count c_var;
  %LET c_count=1;
  %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));

  %DO %WHILE(&c_var ne);
    %LET c_count=%EVAL(&c_count+1);
    %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
  %END;
    %LET c_count=%EVAL(&c_count-1);

  %DO d=1 %TO &c_count;
    %LET c_var=%SCAN(&cvar,&d,%STR( ));

    DATA _trans_;
      SET _trans_;
      trans_&c_var=0;
    RUN;

    /*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

    PROC FREQ DATA=_trans_ NOPRINT;
      TABLES &c_var /  SPARSE OUT=&c_var;
    RUN;

    /*Ausgabedatensatz von PROC FREQ anpassen*/

    DATA &c_var (DROP=percent count);
      SET &c_var;
      IF &c_var=. THEN DELETE;
    RUN;

    /*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/

    %NUMBEROBS(datensatz=&c_var);
    
    %LET counter=&nnnn1;

    /*neue Beobachtungsnummer einfhren*/
    
    DATA &c_var;
      SET &c_var;
      obs_&c_var=_N_;
    RUN;

    %DO f=1 %TO &counter;
      DATA &c_var&f;
        SET &c_var;
        IF obs_&c_var ne &f THEN DELETE;
      RUN;

      DATA &c_var&f;
        SET &c_var&f;
        CALL SYMPUT ('kat_alt',&c_var);
        CALL SYMPUT ('kat_neu',obs_&c_var);
      RUN;

      DATA _trans_;
        SET _trans_;
        IF &c_var=&kat_alt AND trans_&c_var=0 THEN DO;
          &c_var=&kat_neu;
          trans_&c_var=1;
        END;
      RUN;

      /*berflssige Zwischendateien lschen*/

      PROC DATASETS NOLIST;
        DELETE &c_var&f;
      RUN;
      QUIT;
    %END;

    /*jeweilige "MIN-Zahl an String anhngen (immer .)*/

    %LET minlist=&minlist .;

    /*jeweilige "MAX-Zahl an String anhngen (immer .)*/

    %LET maxlist=&maxlist .;


    /*jeweilige "Round- Zahl" an String anhngen (immer 1)*/

    %LET cvar_round=&cvar_round 1;

  %END;
%END;

/*neue Beobachtungsnummer einfhren*/
/***gendert am 17.05.05*/

DATA _tmp2_ (KEEP= &xvar &ovar &nvar _newobs_ &obs_id);
  SET &data;
  _newobs_=_N_;
RUN;

%IF &nomiss_covar=0 AND &range=0 %THEN %DO; /*ergnzt am 6.6.05*/

  DATA _trans_ (KEEP= &xvar &ovar &nvar _newobs_ &obs_id);
    SET _trans_;
    _newobs_=_N_;
  RUN;
%END;

/*PROC MI aufrufen*/
/***ergnzt und gendert am 17.05.05*/

%IF &imputation_art=0 %THEN %DO;
  ODS LISTING CLOSE;
  ODS OUTPUT MissPattern=misspattern; 
%END;

/*PROC MI fr die Ausgabe des Missing Patterns aufgrund von Warn- und
  Fehlermeldungen: 
  HINWEIS: The EM algorithm (MLE) converges in 93 iterations.
  HINWEIS: The EM algorithm (posterior mode) converges in 74 iterations.
  FEHLER: An imputed variable value is not in the specified range after 100 tries.
  HINWEIS: Das SAS System hat die Verarbeitung dieses Schritts aufgrund von Fehlern
         abgebrochen.
  WARNUNG: Die Datei WORK.MISSPATTERN ist mglicherweise unvollstndig. Beim Beenden
           des Schrittes waren 5 Beobachtungen und 9 Variablen vorhanden.
  WARNUNG: Die Datei WORK.OUTMI ist mglicherweise unvollstndig. Beim Beenden des
         Schrittes waren 0 Beobachtungen und 6 Variablen vorhanden.
/*ergnzt am 8.6.05*/

%IF &imputation_art=0 %THEN %DO;
 PROC MI
  DATA=
  %IF &nomiss_covar=0 AND &range=0 %THEN %DO;
    _trans_
  %END;
  %ELSE %DO;
    _tmp2_
  %END;
  ;

  VAR &xvar
    %IF &nomiss_covar=1 %THEN %DO;
      %IF &ovar ne %THEN %DO;
        &ovar
      %END;
      %IF &nvar ne %THEN %DO;
        &nvar
      %END;
    %END;

    %IF &nomiss_covar=0 %THEN %DO;
      &cvar
    %END;
  ; 
  RUN;
%END;
%ELSE %DO;

PROC MI

  DATA=
  %IF &nomiss_covar=0 AND &range=0 %THEN %DO;
    _trans_
  %END;
  %ELSE %DO;
    _tmp2_
  %END;

  OUT=outmi 


%IF &random ne 0 %THEN %DO;
  SEED=&random
%END;


  ROUND=&xvar_round

  %IF &nomiss_covar=0 %THEN %DO;
        &cvar_round
  %END;


  MINIMUM=

  %IF &xvar ne %THEN %DO;  
          &minlistx 
  %END;
  
  %IF &nomiss_covar=0 %THEN %DO; 
          &minlist
  %END;
 

  MAXIMUM=

  %IF &xvar ne %THEN %DO; 
          &maxlistx 
  %END;
  
  %IF &nomiss_covar=0 %THEN %DO; 
          &maxlist
  %END;


%IF &imputation_art ne 0 %THEN %DO;
  NOPRINT
%END;


  NIMPUTE=&nimpute;


/***experimentelles CLASS-Statement*/
%IF &nomiss_covar=1 %THEN %DO;
  CLASS
  %IF &ovar ne %THEN %DO;
        &ovar
  %END;

  %IF &nvar ne %THEN %DO;
        &nvar
  %END;
;
%END;

%IF &miss_patt=0 %THEN %DO;
  %IF %UPCASE(&mi_method)=MCMC %THEN %DO;
    MCMC;
  %END;

  %IF %UPCASE(&mi_method)=EM %THEN %DO;
    EM;
  %END;
%END;


%IF &miss_patt=1 %THEN %DO;
  %IF %UPCASE(&mi_method)=REG OR %UPCASE(&mi_method)=REGRESSION %THEN %DO; 
    MONOTONE REG
    %IF &nomiss_covar=0 %THEN %DO;
      ; /*wichtig, um MONOTONE-Statement zu beenden*/
    %END;
    %IF &nomiss_covar=1 %THEN %DO;
      (&xvar);
    %END;
  %END;
  %IF %UPCASE(&mi_method)=REGPMM OR %UPCASE(&mi_method)=REGPREDMEANMATCH %THEN %DO; 
    MONOTONE REGPMM
    %IF &nomiss_covar=0 %THEN %DO;
      ; /*wichtig, um MONOTONE-Statement zu beenden*/
    %END;
    %IF &nomiss_covar=1 %THEN %DO;
      (&xvar);
    %END;
  %END;
    %IF &nomiss_covar=1 %THEN %DO;
      %IF &ovar ne AND &nvar ne %THEN %DO;
        MONOTONE LOGISTIC(&ovar=&xvar);
        MONOTONE DISCRIM(&nvar=&xvar);
      %END;
      %ELSE %DO;
        %IF &ovar ne AND &xvar ne %THEN %DO;
          MONOTONE LOGISTIC(&ovar=&xvar);
        %END;
        %IF &nvar ne AND &xvar ne %THEN %DO;
          MONOTONE DISCRIM(&nvar=&xvar);
        %END;
      %END;
    %END;
%END;


  VAR &xvar
    %IF &nomiss_covar=1 %THEN %DO;
      %IF &ovar ne %THEN %DO;
        &ovar
      %END;
      %IF &nvar ne %THEN %DO;
        &nvar
      %END;
    %END;

    %IF &nomiss_covar=0 %THEN %DO;
      &cvar
    %END;
; 
RUN;
%END;

%IF &imputation_art=0 %THEN %DO;
  ODS LISTING;
  ODS OUTPUT CLOSE;
%END;

%IF &imputation_art=2 OR &imputation_art=3 %THEN %DO; /*ergnzt am 8.6.05*/

/*Prfen, ob Outmi- Datei Beobachtungen enthlt*/
/*wenn fr ein Datensatz mit einem beliebigen MP eine Methode fr ein monotones
  MP herangezogen wurde*/ /*ergnzt am 23.5.05*/

%LET outmi_check=1;
%IF %UPCASE(&mi_method)=REG OR %UPCASE(&mi_method)=REGRESSION
 OR %UPCASE(&mi_method)=REGPMM OR %UPCASE(&mi_method)=REGPREDMEANMATCH %THEN %DO;
  %NUMBEROBS(datensatz=outmi);
  %LET outmi_check=&nnnn1;
%END;


/*Prfen, ob Outmi- Datei Beobachtungen enthlt, aufgrund folgender Meldung:*/
/*WARNUNG: The EM algorithm (MLE) fails to converge after 200 iterations. You can
           increase the number of iterations (MAXITER= option) or increase the
           value of the convergence criterion (CONVERGE= option).
  HINWEIS: The EM algorithm (posterior mode) converges in XX iterations.
  FEHLER:  An imputed variable value is not in the specified range after 100
           tries.*/ /*ergnzt am 7.6.05 und gendert am 8.6.05*/

%LET check_outmi=-1;

  /*Prfen, wieviele vervollstndigte Datenstze erstellt wurden, auch wenn die
    Fehlermeldung "FEHLER:  An imputed variable value is not in the specified
    range after 100 tries." aufgetreten ist.*/ /*ergnzt am 7.6.05*/

  DATA check_obs;
    SET &data;
    IF &obs_id=. THEN DELETE;
  RUN;

  /*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

  %NUMBEROBS(datensatz=check_obs);

  /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

  %LET obs_data=&nnnn1;
  %LET obs_outmi_soll=%EVAL(&obs_data*&nimpute);

  /*# Beobachtungen der Datei 'outmi' mit Hilfe des Untermakros 'numberobs.mac.sas'
    bestimmen*/

  %NUMBEROBS(datensatz=outmi);

  /*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

  %LET obs_outmi_ist=&nnnn1;

  %IF &obs_outmi_ist=0 %THEN %DO;
    %LET check_outmi=0;
  %END;
  %ELSE %IF &obs_outmi_ist = &obs_outmi_soll %THEN %DO;
    %LET check_outmi=-1;
  %END;
  %ELSE %DO; /*12.7.05*/
    %LET check_outmi=%EVAL(&obs_outmi_ist/&obs_data);
  %END;
  

  /*temporren Datensatz lschen*/

  PROC DATASETS NOLIST;
    DELETE check_obs;
  RUN;
  QUIT;


/*Missing-value-Indikatoren (Variablennamen) als Makro- Variable abspeichern*/

%IF &nomiss_covar=0 AND &range=1 %THEN %DO; /*ergnzt am 6.6.05*/
  %LET liste=;

  %DO h=1 %TO &c_count;
    %LET cvar_liste=%SCAN(&cvar,&h,%STR( ));
  	%LET cvar_liste=&&cvar_liste._missval;
  	%LET liste=&liste &cvar_liste;
  %END;

  /*Missing-value-Indikatoren und neue Beobachtungsnummer zum Mergen behalten*/

  DATA _tmp3_ (KEEP=&liste _newobs_ &obs_id);
    SET _tmp_;
  RUN;

  /*Schleife ber jedes einzelne neue Dataset mit ersetzten missing values*/

  %DO r=1 %TO &nimpute;

    /*Automatische Ausgabedatei von PROC MI aufsplitten*/

    DATA set&r;
      SET outmi;
      IF _IMPUTATION_ ne &r THEN DELETE;
    RUN;

    /*jedes neue Dataset mit zugehrigen missing-value Indikator- Spalten mergen*/

    DATA set&r;
      MERGE _tmp3_ set&r;
      BY _newobs_;
    RUN;

    /*Rcktransformation der Kategorien:*/  

    %DO d=1 %TO &c_count;
      %LET c_var=%SCAN(&cvar,&d,%STR( ));
    
      %NUMBEROBS(datensatz=&c_var); 

      %LET counter=&nnnn1;

      %DO f=1 %TO &counter;
        DATA &c_var&f;
    	  SET &c_var;
    	  IF obs_&c_var NE &f THEN DELETE;
    	RUN;

        /*jeweilige Originalkategorie sowie Ersatzkategorie als Makro- Variable
          abspeichern*/

    	DATA &c_var&f;
    	  SET &c_var&f;
    	  CALL SYMPUT('y',&c_var);
    	  CALL SYMPUT('x', obs_&c_var);
    	RUN;

        /*Ersetzung:*/

    	DATA set&r;
    	  SET set&r;
    	  IF &c_var=&x AND &&c_var._missval=1 THEN DO;
     	    &c_var=&y;
    	    &&c_var._missval=0;
     	  END;
        RUN;

        /*berflssige Zwischendateien lschen*/

    	PROC DATASETS NOLIST;
    	  DELETE &c_var&f ;
    	RUN;
    	QUIT;
      %END;

      PROC DATASETS NOLIST;
        DELETE &c_var&d;
      RUN;
      QUIT;

      /*DM "clear log;";*/
    %END;
  %END;

  /*Ausgabedatenstze der multiplen Imputation anpassen:*/

  %DO z=1 %TO &nimpute;
    DATA &mimputation_file&z (DROP= &liste  _newobs_);
      SET set&z;
    RUN;

    PROC DATASETS NOLIST;
    	DELETE set&z;
    RUN;
    QUIT;
  %END;
%END;

/*Rcktransformation der kategorialen Variablen in ihre ursprnglichen Kategorien
  bzw. Ersetzung der "Ausreier" durch MIN oder MAX*/ /*ergnzt am 6.6.05*/

%IF &nomiss_covar=0 AND &range=0 %THEN %DO; /*ergnzt am 6.6.05*/

  %DO r=1 %TO &nimpute;

    /*Automatische Ausgabedatei von PROC MI aufsplitten*/

    DATA set&r;
      SET outmi;
      IF _IMPUTATION_ ne &r THEN DELETE;
      
    RUN;

    %DO d=1 %TO &c_count;
      %LET c_var=%SCAN(&cvar,&d,%STR( ));

      DATA set&r;
        SET set&r;
        &c_var._trans=0;
      RUN;

      /*Ausprgungen der jeweiligen kategorialen Variable ermitteln*/

      PROC FREQ DATA=_tmpx_ NOPRINT;
        TABLES &c_var /  SPARSE OUT=&c_var;
      RUN;

      /*Ausgabedatensatz von PROC FREQ anpassen*/

      DATA &c_var (DROP=percent count);
        SET &c_var;
        IF &c_var=. THEN DELETE;
      RUN;

      /*Anzahl der mglichen Ausprgungen je kategorialer Variable ermitteln*/
    
   	  %NUMBEROBS(datensatz=&c_var);

	  %LET count_var=&nnnn1;

      /*neue Beobachtungsnummer einfhren*/

      DATA &c_var;
        SET &c_var;
        obs_&c_var=_N_;
      RUN;

      /*alle Beobachtungen bis auf Nr.1 lschen*/

      DATA &c_var&d ;
        SET &c_var;
        IF obs_&c_var ne 1 THEN DELETE;
      RUN;

      /*erste Kategorie als Makro- Variable abspeichern*/

      DATA &c_var&d ;
        SET &c_var&d;
        CALL SYMPUT('min',obs_&c_var);
      RUN;

      /*letzte Kategorie fr Maximum als Makro- Variable abspeichern*/

      DATA max_&c_var;
        SET &c_var;
        IF obs_&c_var ne &count_var THEN DELETE;
      RUN;

      /*letzte Kategorie als Makro- Variable abspeichern*/

      DATA max_&c_var;
        SET max_&c_var;
        CALL SYMPUT('max',obs_&c_var);
      RUN;

      /*Ersetzung der Ausreier durch MINIMUM oder MAXIMUM*/

      DATA set&r;
        SET set&r;
        IF &c_var<=&min AND &c_var ne . THEN
          &c_var=&min;
        IF &c_var>=&max AND &c_var ne . THEN
          &c_var=&max;
      RUN;
        
      
      %DO f=1 %TO &count_var;

        DATA &c_var&f;
    	  SET &c_var;
    	  IF obs_&c_var NE &f THEN DELETE;
    	RUN;

        /*jeweilige Originalkategorie sowie Ersatzkategorie als Makro- Variable
          abspeichern*/
        
        DATA &c_var&f;
          SET &c_var&f;
          CALL SYMPUT ('kat_alt',&c_var);
          CALL SYMPUT ('kat_neu',obs_&c_var);
        RUN;

        /*Ersetzung:*/
        
    	DATA set&r;
    	  SET set&r;
    	  IF &c_var=&kat_neu AND &c_var._trans=0 THEN DO;
     	    &c_var=&kat_alt;
            &c_var._trans=1;
          END;
        RUN;

        /*berflssige Zwischendateien lschen*/
        
    	PROC DATASETS NOLIST;
    	  DELETE &c_var&f max_&c_var;
    	RUN;
    	QUIT;
      %END;

      DATA set&r (DROP=&c_var._trans);
        SET set&r;
      RUN;
    %END;
  %END;

  %DO z=1 %TO &nimpute;
    DATA &mimputation_file&z (DROP=_newobs_);
      SET set&z;
    RUN;
    
    PROC DATASETS NOLIST;
    	DELETE set&z;
    RUN;
    QUIT;
  %END;
%END;


/**ergnzt am 23.5.05 durch KB*/

%IF &nomiss_covar=1 %THEN %DO;
  %DO z=1 %TO &nimpute;
    DATA &mimputation_file&z (DROP=_newobs_);
      SET outmi;
      IF _IMPUTATION_=&z;
    RUN;
  %END;
%END;


%IF &nimpute >1 %THEN %DO;

  /*Ausgabe der multiplen Imputation:*/

  DATA null2;
    impute="&mimputation_file.1 - &mimputation_file&nimpute";
  RUN;
%END;

%ELSE %DO;

  DATA null2;
    impute="&mimputation_file.1";
  RUN;
%END;

  %END;
%END;
%END; /*ergnzt am 8.6.05*/


/**********************************/
/****Ausgabe ins OUTPUT-Fenster****/
/**********************************/

/*Single Imputation*/

%IF &imputation_art ne 0 %THEN %DO;
  %IF &imputation_art ne 2 %THEN %DO;

/*--------------------*/

/*bersicht ber die Missing- Value- Situation im angegebenen Datensatz*/
/*im Output- Fenster ausgeben:*/

%IF &cvar ne %THEN %DO;

  PROC PRINT DATA=var_uebersicht NOOBS LABEL;
    TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	TITLE3 "Anzahl Beobachtungen: &data_count";
	TITLE4 'Missing Value- Indikator: 1=ja / 0=nein';
	TITLE5 ;
	TITLE6 'Deskription der durchgefhrten Single- Imputation';

    VAR variable missing_kategorie miss_indi;

    LABEL variable='kategorielle Variable';
	LABEL missing_kategorie='zugehrige Referenzkategorie fr missing values';
	LABEL miss_indi='Missing Value- Indikator';
   RUN;
%END;

%IF &xvar ne %THEN %DO;

  PROC PRINT DATA=var_uebersicht2 NOOBS LABEL;

    %IF &cvar ne %THEN %DO;

	  TITLE1 ;
	  TITLE2 ;
	  TITLE3 ;
	  TITLE4 ;
	  TITLE5 ;
	  TITLE6 ;
	%END;

    %ELSE %DO;

	  TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	  TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	  TITLE3 "Anzahl Beobachtungen: &data_count";
	  TITLE4 'Missing Value- Indikator: 1=ja / 0=nein';
	  TITLE5 ;
	  TITLE6 'Deskription der durchgefhrten Single- Imputation';
	%END;

 	  VAR variable2 replace_method miss_indi;

      LABEL variable2='stetige Variable';
	  LABEL replace_method='zugehrige Ersetzungsmethode';
	  LABEL miss_indi='Missing Value- Indikator';
  RUN;
%END;

PROC PRINT DATA=null NOOBS LABEL;
  TITLE 'Single Imputation';
  VAR ersetz;
  LABEL ersetz='Ergebnisdatensatz mit der missing value- Ersetzung:';
RUN;

%END;

%IF &category_test=1 %THEN %DO;

  DATA category_test;
    var="&category_test_fail";
  RUN;

  %IF &imputation_art ne 2 %THEN %DO;

    PROC PRINT DATA=category_test NOOBS LABEL;
	  TITLE1 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method";
	  TITLE2 ;
	  TITLE3 'Es konnte keine Multiple Imputation durchgefhrt werden,';
	  TITLE4 'da die folgenden Variablen lediglich eine Ausprgung besitzen!';
	  TITLE5 'Etwaige fehlende Werte knnen nicht geschtzt werden';

      VAR var;

      LABEL var='Variable';
	RUN;
  %END;

  %ELSE %DO;
    %IF &nomiss_covar=1 AND &ovar ne AND &nvar eq %THEN %DO;
    /*ergnzt am 30.5.05*/

      PROC PRINT DATA=category_test NOOBS LABEL;
	    TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	    TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	    TITLE3 "Anzahl Beobachtungen: &data_count";
	    TITLE5 ;
	    TITLE6 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method, LOGISTIC";
	    TITLE7 ;
	    TITLE8 'Es konnte keine Multiple Imputation durchgefhrt werden,';
	    TITLE9 'da die folgenden Variablen lediglich eine Ausprgung besitzen!';
	    TITLE10 'Etwaige fehlende Werte knnen nicht geschtzt werden';

        VAR var;

        LABEL var='Variable';
	  RUN;
    %END;
    %ELSE %IF &nomiss_covar=1 AND &nvar ne AND &ovar eq %THEN %DO;
      PROC PRINT DATA=category_test NOOBS LABEL;
	    TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	    TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	    TITLE3 "Anzahl Beobachtungen: &data_count";
	    TITLE5 ;
	    TITLE6 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method, DISCRIM";
	    TITLE7 ;
	    TITLE8 'Es konnte keine Multiple Imputation durchgefhrt werden,';
	    TITLE9 'da die folgenden Variablen lediglich eine Ausprgung besitzen!';
	    TITLE10 'Etwaige fehlende Werte knnen nicht geschtzt werden';

        VAR var;

        LABEL var='Variable';
	  RUN;
    %END;
    %ELSE %IF &nomiss_covar=1 AND &ovar ne AND &nvar ne %THEN %DO;
      PROC PRINT DATA=category_test NOOBS LABEL;
	    TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	    TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	    TITLE3 "Anzahl Beobachtungen: &data_count";
	    TITLE5 ;
	    TITLE6 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method, LOGISTIC/DISCRIM";
	    TITLE7 ;
	    TITLE8 'Es konnte keine Multiple Imputation durchgefhrt werden,';
	    TITLE9 'da die folgenden Variablen lediglich eine Ausprgung besitzen!';
	    TITLE10 'Etwaige fehlende Werte knnen nicht geschtzt werden';

        VAR var;

        LABEL var='Variable';
	  RUN;
    %END;
    %ELSE %DO;

      PROC PRINT DATA=category_test NOOBS LABEL;
	    TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	    TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	    TITLE3 "Anzahl Beobachtungen: &data_count";
	    TITLE5 ;
	    TITLE6 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method";
	    TITLE7 ;
	    TITLE8 'Es konnte keine Multiple Imputation durchgefhrt werden,';
	    TITLE9 'da die folgenden Variablen lediglich eine Ausprgung besitzen!';
	    TITLE10 'Etwaige fehlende Werte knnen nicht geschtzt werden';

        VAR var;

        LABEL var='Variable';
	  RUN;
    %END;
  %END;

  PROC DATASETS NOLIST;
    DELETE category_test;
  RUN;
  QUIT;
%END;



%IF &category_test=0 %THEN %DO;
  %IF &imputation_art ne 1 %THEN %DO;

/*--------------------*/

%IF &imputation_art=3 %THEN %DO;

  PROC PRINT DATA=null2 NOOBS LABEL;
    TITLE1 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method";
	TITLE2 "Anzahl der Imputationen: &nimpute";
	TITLE3 ;
	TITLE4 ;

    VAR impute;

    LABEL impute='Ergebnisdatenstze der multiplen Imputation:';
  RUN;

  %IF &outmi_check=0 %THEN %DO;
    /*ergnzt am 30.5.05, gendert am 8.6.05*/

    DATA outmi_check;
	  outmi_check="";
    RUN;

    PROC PRINT DATA=outmi_check NOOBS LABEL;
	  TITLE1 "Bei der multiplen Imputation mit der Methode &mi_method";
	  TITLE2 'ist ein Fehler aufgetreten!';
	  TITLE3 "Mgliche Ursache: MI-Methode '&mi_method' wurde im Zusammengang";
      TITLE4 'mit einem Datensatz gewhlt, der ein nicht-monotones missing pattern';
      TITLE5 'besitzt! O D E R: Nach 100 Ersetzungsversuchen befand sich der zu';
      TITLE6 'ersetzende Wert auerhalb des Wertebereiches der Variablen.';
      TITLE7 'Um dieses Problem zu umgehen, knnen Sie im Aufruf dem Makro bergeben,';
      TITLE8 'dass kein Wertebereich (range=0) bestimmt werden soll.';
      TITLE9 'Genauere Information knnen evtl. dem LOG- Fenster';
	  TITLE10 'entnommen werden.';
      LABEL outmi_check='--------------------------------------------------';
    RUN;

    PROC DATASETS NOLIST;
      DELETE outmi_check;
    RUN;
    QUIT;
  %END;

  /*ergnzt am 7.6.05*/
  %IF &check_outmi=0 AND &outmi_check=1 %THEN %DO;

    DATA check_outmi;
	  check_outmi="";
    RUN;

    PROC PRINT DATA=check_outmi NOOBS LABEL;
	  TITLE1 "Bei der multiplen Imputation mit der Methode &mi_method";
	  TITLE2 'ist ein Fehler aufgetreten!';
	  TITLE3 "Mgliche Ursache: Nach 100 Ersetzungsversuchen befand sich der zu";
      TITLE4 'ersetzende Wert auerhalb des Wertebereiches der Variablen.';
      TITLE5 'Um dieses Problem zu umgehen, knnen Sie im Aufruf dem Makro bergeben,';
      TITLE6 'dass kein Wertebereich (range=0) bestimmt werden soll.';
      TITLE8 "Die erstellten Datenstze enthalten keine Obseravtionen!";
	  TITLE9 'Genauere Information knnen evtl. dem LOG- Fenster';
	  TITLE10 'entnommen werden.';
	  LABEL check_outmi='--------------------------------------------------';
    RUN;

    PROC DATASETS NOLIST;
	  DELETE check_outmi;
    RUN;
    QUIT;
  %END;

  /*ergnzt am 8.6.05*/
  %IF &outmi_check=1 AND &check_outmi>=1 AND &check_outmi<&nimpute %THEN %DO;

    DATA check_outmi;
	  check_outmi="";
    RUN;

    PROC PRINT DATA=check_outmi NOOBS LABEL;
	  TITLE1 "Bei der multiplen Imputation mit der Methode &mi_method";
	  TITLE2 'ist ein Fehler aufgetreten!';
	  TITLE3 "Mgliche Ursache: Nach 100 Ersetzungsversuchen befand sich der zu";
      TITLE4 'ersetzende Wert auerhalb des Wertebereiches der Variablen.';
      TITLE5 'Um dieses Problem zu umgehen, knnen Sie im Aufruf dem Makro bergeben,';
      TITLE6 'dass kein Wertebereich (range=0) bestimmt werden soll.';
      TITLE8 "Es wurden nur &check_outmi vervollstndigte Datenstze erstellt!";
	  TITLE9 'Genauere Information knnen evtl. dem LOG- Fenster';
	  TITLE10 'entnommen werden.';
	  LABEL check_outmi='--------------------------------------------------';
    RUN;

    PROC DATASETS NOLIST;
	  DELETE check_outmi;
    RUN;
    QUIT;
  %END;
%END;

%ELSE %DO;
  %IF &nomiss_covar=1 AND &ovar ne AND &nvar eq %THEN %DO;
  /*ergnzt am 30.5.05*/

    PROC PRINT DATA=null2 NOOBS LABEL;
	  TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	  TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	  TITLE3 "Anzahl Beobachtungen: &data_count";
	  TITLE4 ;
	  TITLE5 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method, LOGISTIC";
	  TITLE6 "Anzahl der Imputationen: &nimpute";
	  TITLE7 ;
	  TITLE8 ;

      VAR impute;

      LABEL impute='Ergebnisdatenstze der multiplen Imputation:';
    RUN;
  %END;
  %ELSE %IF &nomiss_covar=1 AND &nvar ne AND &ovar eq %THEN %DO; 
  /*ergnzt am 30.5.05*/
    PROC PRINT DATA=null2 NOOBS LABEL;
	  TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	  TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	  TITLE3 "Anzahl Beobachtungen: &data_count";
	  TITLE4 ;
	  TITLE5 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method, DISCRIM";
	  TITLE6 "Anzahl der Imputationen: &nimpute";
	  TITLE7 ;
	  TITLE8 ;

      VAR impute;

      LABEL impute='Ergebnisdatenstze der multiplen Imputation:';
    RUN;
  %END;
  %ELSE %IF &nomiss_covar=1 AND &ovar ne AND &nvar ne %THEN %DO;
    /*ergnzt am 30.5.05*/
    PROC PRINT DATA=null2 NOOBS LABEL;
	  TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	  TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	  TITLE3 "Anzahl Beobachtungen: &data_count";
	  TITLE4 ;
	  TITLE5 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method, LOGISTIC/DISCRIM";
	  TITLE6 "Anzahl der Imputationen: &nimpute";
	  TITLE7 ;
	  TITLE8 ;

      VAR impute;

      LABEL impute='Ergebnisdatenstze der multiplen Imputation:';
    RUN;
  %END;
  %ELSE %DO;
    PROC PRINT DATA=null2 NOOBS LABEL;
	  TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen Datensatz';
	  TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	  TITLE3 "Anzahl Beobachtungen: &data_count";
	  TITLE4 ;
	  TITLE5 "Multiple Imputation mittels SAS- Prozedur PROC MI. Methode: &mi_method";
	  TITLE6 "Anzahl der Imputationen: &nimpute";
	  TITLE7 ;
	  TITLE8 ;

      VAR impute;

      LABEL impute='Ergebnisdatenstze der multiplen Imputation:';
    RUN;
  %END;

  /*ergnzt am 30.5.05, gendert am 8.6.05*/
  %IF &outmi_check=0 %THEN %DO;

    DATA outmi_check;
	  outmi_check="";
    RUN;

    PROC PRINT DATA=outmi_check NOOBS LABEL;
	  TITLE1 "Bei der multiplen Imputation mit der Methode &mi_method";
	  TITLE2 'ist ein Fehler aufgetreten!';
	  TITLE3 "Mgliche Ursache: MI-Methode '&mi_method' wurde im Zusammengang";
      TITLE4 'mit einem Datensatz gewhlt, der ein nicht-monotones missing pattern';
      TITLE5 'besitzt! O D E R: Nach 100 Ersetzungsversuchen befand sich der zu';
      TITLE6 'ersetzende Wert auerhalb des Wertebereiches der Variablen.';
      TITLE7 'Um dieses Problem zu umgehen, knnen Sie im Aufruf dem Makro bergeben,';
      TITLE8 'dass kein Wertebereich (range=0) bestimmt werden soll.';
      TITLE9 'Genauere Information knnen evtl. dem LOG- Fenster';
	  TITLE10 'entnommen werden.';
      LABEL outmi_check='--------------------------------------------------';
    RUN;

    PROC DATASETS NOLIST;
	  DELETE outmi_check;
    RUN;
    QUIT;
  %END;

  /*ergnzt am 7.6.05*/
  %IF &check_outmi=0 AND &outmi_check=1 %THEN %DO;
   
    DATA check_outmi;
	  check_outmi="";
    RUN;

    PROC PRINT DATA=check_outmi NOOBS LABEL;
	  TITLE1 "Bei der multiplen Imputation mit der Methode &mi_method";
	  TITLE2 'ist ein Fehler aufgetreten!';
	  TITLE3 "Mgliche Ursache: Nach 100 Ersetzungsversuchen befand sich der zu";
      TITLE4 'ersetzende Wert auerhalb des Wertebereiches der Variablen.';
      TITLE5 'Um dieses Problem zu umgehen, knnen Sie im Aufruf dem Makro bergeben,';
      TITLE6 'dass kein Wertebereich (range=0) bestimmt werden soll.';
      TITLE8 "Die erstellten Datenstze enthalten keine Obseravtionen!";
	  TITLE9 'Genauere Information knnen evtl. dem LOG- Fenster';
	  TITLE10 'entnommen werden.';
	  LABEL check_outmi='--------------------------------------------------';
    RUN;

    PROC DATASETS NOLIST;
	  DELETE check_outmi;
    RUN;
    QUIT;
  %END;

  /*ergnzt am 8.6.05*/
  %IF &outmi_check=1 AND &check_outmi>=1 AND &check_outmi<&nimpute %THEN %DO;

    DATA check_outmi;
	  check_outmi="";
    RUN;

    PROC PRINT DATA=check_outmi NOOBS LABEL;
	  TITLE1 "Bei der multiplen Imputation mit der Methode &mi_method";
	  TITLE2 'ist ein Fehler aufgetreten!';
	  TITLE3 "Mgliche Ursache: Nach 100 Ersetzungsversuchen befand sich der zu";
      TITLE4 'ersetzende Wert auerhalb des Wertebereiches der Variablen.';
      TITLE5 'Um dieses Problem zu umgehen, knnen Sie im Aufruf dem Makro bergeben,';
      TITLE6 'dass kein Wertebereich (range=0) bestimmt werden soll.';
      TITLE8 "Es wurden nur &check_outmi vervollstndigte Datenstze erstellt!";
	  TITLE9 'Genauere Information knnen evtl. dem LOG- Fenster';
	  TITLE10 'entnommen werden.';
	  LABEL check_outmi='--------------------------------------------------';
    RUN;

    PROC DATASETS NOLIST;
	  DELETE check_outmi;
    RUN;
    QUIT;
  %END;
%END;
  %END;
%END;

%END;

%ELSE %DO;

  DATA misspattern (DROP=&xvar &cvar);
	SET misspattern;
  RUN;

  PROC PRINT DATA=misspattern NOOBS LABEL;
    TITLE1 'bersicht ber die missing value-Struktur im angegebenen Datensatz';
	TITLE2 "&fallzahldata (&percent_miss %) aller Beobachtungen enthielten fehlende Werte";
	TITLE3 "Anzahl Beobachtungen: &data_count";
	TITLE4 "'X' = Wert vorhanden / '.' = missing / 'O' = missing, wird nicht ersetzt";
	TITLE5 ;
	TITLE6 'Missing Data Patterns';
  RUN;

  PROC DATASETS NOLIST;
	DELETE misspattern;
  RUN;
  QUIT;
%END;

/*berflssige Zwischendateien lschen:*/

PROC DATASETS NOLIST;
  %IF &imputation_art ne 0 %THEN %DO;
   %IF &imputation_art ne 2 %THEN %DO;
     DELETE var_uebersicht var_uebersicht2  null nomiss ;
   %END;
  %END;
  %IF &category_test=0 %THEN %DO;
    %IF &imputation_art ne 1 %THEN %DO;
	  DELETE _tmp3_ _tmp2_ outmi _trans_ &cvar null2 mi;
    %END;
  %END;
  DELETE _tmpx_ _tmp_ 
  %IF &imputation_art = 0 %THEN %DO;
    round nomiss
  %END;
  %IF &imputation_art = 2 %THEN %DO;
    round nomiss
  %END;
;
RUN;
QUIT;


%nomissing:

%IF &miss=0 %THEN %DO;

  DATA null;
	ersetz='';
  RUN;


  /*bersicht ber die Missing- Value- Situation im angegebenen Datensatz*/
  /*im Output- Fenster ausgeben:*/

  PROC PRINT DATA=null NOOBS LABEL;
	TITLE1 'bersicht ber die Imputation fehlender Werte im angegebenen';
	TITLE2 "Datensatz [Keine Beobachtungen enthielten fehlende Werte!]";
	TITLE3 "Anzahl Beobachtungen: &data_count";
	TITLE4 'Es wurden keine (multiplen) Imputationen ausgefhrt!';

    VAR ersetz;

    LABEL ersetz='_______________________________________';
  RUN;

  PROC DATASETS NOLIST;
	DELETE null _tmp_ _tmpx_ round nomiss;
  RUN;
  QUIT;
%END;


/*Sprungmarke im Fehlerfall*/

%ende:

%MEND MISSING;
%MISSING(data=work.expdel,xvar=value,ovar=id time,obs_id=beob,miss_patt=0,imputation_art=2,mi_method=EM,xvar_round=0.01,nimpute=5,mimputation_file=work.mimputed,macro_path='F:/Promotion/Programme');
run;
